<?php
/**
 * Reference: https://developer.wordpress.org/plugins/settings/custom-settings-page
 */

namespace CarDealerPress\Includes\Admin;

use CarDealerPress\Templates\Admin as templates;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * @internal never define functions inside callbacks.
 * these functions could be run multiple times; this would result in a fatal error.
 */

/**
 * custom option and settings
 */
class Shortcode_Settings_Page {


	public function __construct() {
		/*
		 * Register our settings_init to the admin_init action hook.
		 */
		add_action( 'admin_init', array( $this, 'settings_init' ) );
		/*
		 * Register our admin_options_page to the admin_menu action hook.
		 */
		add_action( 'admin_menu', array( $this, 'options_page' ) );
		$this->load_admin_templates();

	}//end __construct()

	public function load_admin_templates() {
		foreach ( glob( CDP_DIR_PATH . 'templates/admin/*/*.php' ) as $file ) {
			include_once $file;
		}

	}


	/**
	 * Custom option and settings:
	 *  - callback functions
	 */

	public function settings_init() {
		// Register a new setting for "cdp admin" page.
		register_setting( 'cdp_sc_group', 'cdp_sc_options' );

		// Register a new section in the "cdp admin" page.
		add_settings_section(
			'cdp_admin',
			__( '', 'cdp_admin' ),
			array(
				$this,
				'section_developers_callback',
			),
			'cdp_sc_group' // slug name of page to show up on
		);

		// Register a new field in the "wporg_section_developers" section, inside the "wporg" page.
		// not sure how to use fields, skipping for now
		/*
			add_settings_field(
				'admin_fields', // As of WP 4.6 this value is used only internally.
				// Use $args' label_for to populate the id inside the callback.
				__( 'Pill', 'wporg' ),
				array($this,'admin_field_cb'),
				'wporg',
				'wporg_section_developers',
				array(
					'label_for'         => 'wporg_field_pill',
					'class'             => 'wporg_row',
					'wporg_custom_data' => 'custom',
				)
			);*/

	}//end settings_init()

	/**
	 * Developers section callback function.
	 *
	 * @param   array $args  The settings array, defining title, id, callback.
	 */
	public function section_developers_callback( $args ) {
		$default_tab = 'inventory_detail';
		$tab         = ( $_GET['tab'] ?? $default_tab );
		$content     = '<div id="cdp-content-wrapper-default" class="settings-wrapper display">';
		$content    .= '<div class="wrap">';
		$content    .= '<nav class="nav-tab-wrapper">';
		$content    .= '<a href="?page=cardealerpress_shortcodes&tab=inventory_detail" class="nav-tab ' . ( 'inventory_detail' === $tab ? 'nav-tab-active' : '' ) . '">Inventory Detail</a>';
		$content    .= '<a href="?page=cardealerpress_shortcodes&tab=inventory_slider" class="nav-tab ' . ( 'inventory_slider' === $tab ? 'nav-tab-active' : '' ) . '">Inventory Slider</a>';
		$content    .= '<a href="?page=cardealerpress_shortcodes&tab=inventory_list"	class="nav-tab ' . ( 'inventory_list' === $tab ? 'nav-tab-active' : '' ) . '">Inventory List</a>';
		$content    .= '<a href="?page=cardealerpress_shortcodes&tab=inventory_counter" class="nav-tab ' . ( 'inventory_counter' === $tab ? 'nav-tab-active' : '' ) . '">Inventory Counter</a>';
		$content    .= '<a href="?page=cardealerpress_shortcodes&tab=inventory_search"	class="nav-tab ' . ( 'inventory_search' === $tab ? 'nav-tab-active' : '' ) . '">Inventory Search</a>';
		$content    .= '</nav>';
		$content    .= '<div class="admin-tab-content">';
		switch ( $tab ) :
			case 'inventory_slider':
				$content .= templates\Shortcode\inventory_slider();
				break;

			case 'inventory_list':
				$content .= templates\Shortcode\inventory_list();
				break;

			case 'inventory_counter':
				$content .= templates\Shortcode\inventory_counter();
				break;

			case 'inventory_search':
				$content .= templates\Shortcode\inventory_search();
				break;

			default:
				$content .= templates\Shortcode\inventory_detail();
				break;
		endswitch;
		$content .= '</div>';
		$content .= '</div>';
		echo $content;

	}//end section_developers_callback()

	/**
	 * Add the top level menu page.
	 */
	public function options_page() {
		add_submenu_page(
			'cardealerpress_admin',
			'CarDealerPress - ShortCodes',
			'ShortCodes',
			'manage_options',
			'cardealerpress_shortcodes',
			array(
				$this,
				'options_page_html',
			)
		);

	}//end options_page()

	/**
	 * Top level menu callback function
	 */
	public function options_page_html() {
		// check user capabilities
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		// add error/update messages
		// check if the user have submitted the settings
		// WordPress will add the "settings-updated" $_GET parameter to the url
		if ( isset( $_GET['settings-updated'] ) ) {
			// add settings saved message with the class of "updated"
			add_settings_error( 'cdp_messages', 'cdp_message', __( 'Settings Saved', 'cdp_sc_group' ), 'updated' );
		}

		// show error/update messages
		settings_errors( 'cdp_messages' );
		?>
		<div class="wrap">
			<h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
			<form action="options.php" method="post">
				<?php
				// output security fields for the registered setting "wporg"
				// settings_fields( 'cdp_admin_group' ); //not using fields
				// output setting sections and their fields
				// (sections are registered for "wporg", each field is registered to a specific section)
				do_settings_sections( 'cdp_sc_group' );
				// output save settings button
				// submit_button( 'Save Settings' ); no need for a submit button on shortcode page, information only.
				?>
			</form>
		</div>
		<?php

	}//end options_page_html()


}//end class

