<?php
/**
 * Cron Class File
 *
 * This class defines tasks that must be run when.
 */

namespace CarDealerPress\Includes\Cron;

use CarDealerPress\includes\functions\Logger;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Cron {
	use Logger;


	public function remove_cron_schedule() {
		$blog_id   = get_current_blog_id();
		$timestamp = wp_next_scheduled( 'cdp_cron_inventory_update' );
		wp_unschedule_event( $timestamp, 'cdp_cron_inventory_update' );
		$this->append_log( array( 'message' => 'CDP Message: WP Cron cdp_cron_inventory_update removed, on site:' . $blog_id ) );
	}

	public function init_refresh_log_file() {
		add_action( 'cdp_cron_log_file_refresh', array( $this, 'refresh_log_file' ) );
		$this->add_wp_cron_log_job();
	}

	function add_wp_cron_log_job() {
		if ( ! wp_next_scheduled( 'cdp_cron_log_file_refresh' ) ) {
			$this->append_log( array( 'message' => 'CDP Cron Job Created - Log file' ) );
			wp_schedule_event( time(), 'daily', 'cdp_cron_log_file_refresh' ); // or run twicedaily.
		}
	}
	public function refresh_log_file() {
		$log_file_creation = filectime( CDP_LOGS );
		$now               = time();
		// Start with keeping 24 hours of logs.
		$time_for_refresh = 60 * 60 * 24;
		#if ( $now - $log_file_creation >= $time_for_refresh ) {
			unlink( CDP_LOGS );
			touch( CDP_LOGS );
			$this->append_log( array( 'message' => 'Log file created' ) );
		#}
	}
}
