<?php

namespace CarDealerPress\Includes\Database;

use CarDealerPress\includes\functions\Logger;
use CarDealerPress\Includes\Helpers\Option;
use CarDealerPress\Includes\Init\Default_Options;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
class Alter_Tables {
	use Logger;

	private static $blog;
	public $current_inventory_version;
	public $default_current_company_version;
	public $current_company_version;
	public $initial_company_version;
	public $initial_inventory_version;
	public static $wpdb;
	/**
	 * @var false|mixed|void
	 */
	public $admin_options;
	/**
	 * @var mixed
	 */
	private $default_current_inventory_version;
	// public $logger;
	public $force;

	public function __construct( $args = array() ) {
		global $wpdb;
		self::$wpdb                              = $wpdb;
		self::$blog                              = get_current_blog_id();
		$this->admin_options                     = Option::get_cdp_admin_options();
		$this->initial_company_version           = Option::check_for_option( 'cdp_admin_options', array(
				'admin',
				'database',
				'company',
				'initial',
			) );
		$this->initial_inventory_version         = Option::check_for_option( 'cdp_admin_options', array(
				'admin',
				'database',
				'inventory',
				'initial',
			) );
		$this->current_company_version           = Option::check_for_option( 'cdp_admin_options', array(
				'admin',
				'database',
				'company',
				'current',
			) );
		$this->current_inventory_version         = Option::check_for_option( 'cdp_admin_options', array(
				'admin',
				'database',
				'inventory',
				'current',
			) );
		$this->default_current_company_version   = Default_Options::$default_options_admin['admin']['database']['company']['current'];
		$this->default_current_inventory_version = Default_Options::$default_options_admin['admin']['database']['inventory']['current'];
		$this->force                             = isset( $args['force'] ) ? true : '';
		$this->company_table();
		$this->inventory_table();
		update_option( 'cdp_admin_options', $this->admin_options );
	}

	function company_table() {
		$table_main = 'cdp_companies';

		if ( $this->force || ( version_compare( $this->default_current_company_version, $this->current_company_version, '>' ) && self::$wpdb->get_var( "SHOW TABLES LIKE '$table_main'" ) === $table_main ) ) {
			if ( $this->force ) {
				$this->append_log( array( 'message' => 'CDP Message: Checking Companies table for changes.' ) );
			} else {
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated to: ' . $this->default_current_company_version . ' | from: ' . $this->default_current_company_version ) );
			}
			/***
			 * Added 08/31/2018 column: id  | DB Version 4.9.180800
			 */
			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'id'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies ADD COLUMN id bigint(20) NOT NULL AUTO_INCREMENT PRIMARY KEY' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated added column id, ' . $this->return_result( $result ) ) );
			}

			/***
			 * Added 01/09/2019 column: inventory_api_last_modified  | DB Version 4.9.180800
			 */
			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'inventory_api_last_modified'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies ADD COLUMN inventory_api_last_modified int(11) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated added column inventory_api_last_modified, ' . $this->return_result( $result ) ) );
			}

			/***
			 * Added 07/29/2020 field: apikeys (AIS related) | DB Version 5.4.200704
			 */
			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'apikeys'" );
			if ( ! empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies DROP COLUMN apikeys' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated removed column apikeys, ' . $this->return_result( $result ) ) );
			}

			/***
			 * Added 06/2022 fields: inventory_last_checked, inventory_last_checked_response added 06/2022 | DB Version 6.0.220600
			 */
			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'inventory_last_checked'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies ADD COLUMN inventory_last_checked int(11) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated added column inventory_last_checked, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies ADD COLUMN inventory_last_checked_response int(11) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated added column inventory_last_checked_response, ' . $this->return_result( $result ) ) );
			}
			/***
			 * Added 09/2022  fields: inventory_last_checked, inventory_last_checked_response | DB Version 6.0.2209.01
			 */
			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'seo'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies ADD COLUMN seo longtext NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated added column seo, ' . $this->return_result( $result ) ) );
			}

//			/***
//			 * Added 12/2022 column rename  | DB Version 6.1.2212.00
//			 */
//			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'country'" );
//			if ( $exists ) {
//				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies RENAME COLUMN country TO country_code' );
//				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated renamed country column to country_code, ' . $this->return_result( $result ) ) );
//			}

			/***
			 * Added 12/2023 country column refresh  | DB Version 6.4.2312.00
			 * Some sites are not renaming the country column.
			 */
			$exists = self::$wpdb->get_var( "SHOW COLUMNS FROM cdp_companies like 'country'" );
			if ( $exists ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies DROP COLUMN country' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated DROPPED country column, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_companies ADD COLUMN country_code varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Company DB Updated ADDED country_code column, ' . $this->return_result( $result ) ) );
			}

			$this->admin_options['admin']['database']['company']['current'] = $this->default_current_company_version;
		}
	}

	function inventory_table() {

		$table_main   = 'cdp_inventory_' . self::$blog;
		$table_import = 'cdp_inventory_import_' . self::$blog;
		if ( $this->force || ( version_compare( $this->default_current_inventory_version, $this->current_inventory_version, '>' ) && self::$wpdb->get_var( "SHOW TABLES LIKE '$table_main'" ) == $table_main ) ) {
			if ( $this->force ) {
				$this->append_log( array( 'message' => 'CDP Message: Checking Inventory table for changes.' ) );
			} else {
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated to: ' . $this->default_current_inventory_version . ' | from: ' . $this->current_inventory_version ) );
			}
			/***
			 * Added 02/12/2018 column: pricesort | DB Version 4.9.180200
			 */
			$exists = self::$wpdb->get_var( 'SHOW COLUMNS FROM cdp_inventory_import_' . self::$blog . " like 'pricesort'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' ADD COLUMN pricesort INT DEFAULT 0' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated added column pricesort, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' ADD COLUMN pricesort INT DEFAULT 0' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated added column pricesort, ' . $this->return_result( $result ) ) );
			}

			/***
			 * Added 04/17/2018 column: color_map | DB Version 4.9.180400
			 */
			$exists = self::$wpdb->get_var( 'SHOW COLUMNS FROM cdp_inventory_import_' . self::$blog . " like 'color_map'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' ADD COLUMN color_map varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated added column color_map, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' ADD COLUMN color_map varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated added column color_map, ' . $this->return_result( $result ) ) );
			}
			/***
			 * Update 02/26/2019 column: id data type | DB Version 5.1.190200
			 */
			$exists = self::$wpdb->get_var( "SELECT DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = 'cdp_inventory_" . self::$blog . "' AND COLUMN_NAME = 'id'" );
			if ( ! empty( $exists ) && $exists == 'mediumint' ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' MODIFY COLUMN id bigint(20) NOT NULL AUTO_INCREMENT' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated altered id data type, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' MODIFY COLUMN id bigint(20) NOT NULL AUTO_INCREMENT' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated altered id data type, ' . $this->return_result( $result ) ) );
			}
			/***
			 * Added 04/20/2019 column: color_map_int | DB Version 5.1.190400
			 */
			$exists = self::$wpdb->get_var( 'SHOW COLUMNS FROM cdp_inventory_import_' . self::$blog . " like 'color_map_int'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' ADD COLUMN color_map_int varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated added column color_map_int, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' ADD COLUMN color_map_int varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated added column color_map_int, ' . $this->return_result( $result ) ) );
			}
			/***
			 * Added 08/28/2019 column: carfax_image_url | DB Version 5.2.190800
			 */
			$exists = self::$wpdb->get_var( 'SHOW COLUMNS FROM cdp_inventory_import_' . self::$blog . " like 'carfax_image_url'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' ADD COLUMN carfax_image_url varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated added column carfax_image_url, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' ADD COLUMN carfax_image_url varchar(255) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated added column carfax_image_url, ' . $this->return_result( $result ) ) );
			}
			/***
			 * Added 01/20/2020 column: thumbnail increase varchar | DB Version 5.3.200101
			 * due to stock image url length
			 */
			$col_info_thumbnail = self::$wpdb->get_col_length( $table_import, 'thumbnail' );
			if ( 350 < $col_info_thumbnail['length'] ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' MODIFY thumbnail varchar(350) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Update column thumbnail varchar length, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' MODIFY thumbnail varchar(350) NOT NULL' );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Update column thumbnail varchar length, ' . $this->return_result( $result ) ) );
			}
			/***
			 * Added 03/03/2022 column: sold_on | DB Version 5.9.2203.00
			 */
			$exists = self::$wpdb->get_var( 'SHOW COLUMNS FROM cdp_inventory_import_' . self::$blog . " like 'sold_on'" );
			if ( empty( $exists ) ) {
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . " ADD COLUMN sold_on varchar(10) DEFAULT ''" );
				$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated added column sold_on, ' . $this->return_result( $result ) ) );
				$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . " ADD COLUMN sold_on varchar(10) DEFAULT ''" );
				$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated added column sold_on, ' . $this->return_result( $result ) ) );
			}
				/***
				 * Added 01/25/2023 column: tags increase varchar  | DB Version 6.1.230100
				 */
				$col_info_tags = self::$wpdb->get_col_length( $table_import, 'tags' );
				if ( 350 < $col_info_tags['length'] ) {
					$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' MODIFY tags varchar(350) NOT NULL' );
					$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Update column tags varchar length, ' . $this->return_result( $result ) ) );
					$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' MODIFY tags varchar(350) NOT NULL' );
					$this->append_log( array( 'message' => 'CDP Message: Inventory DB Update column tags varchar length, ' . $this->return_result( $result ) ) );
				}
				/***
				 * Added 09/01/2023 column: vehicle_url | DB Version 6.2.2309.00
				 * Due to migration from api to db.
				 */
				$exists = self::$wpdb->get_var( 'SHOW COLUMNS FROM cdp_inventory_import_' . self::$blog . " like 'vehicle_url'" );
				if ( empty( $exists ) ) {
					$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_import_' . self::$blog . ' ADD COLUMN vehicle_url varchar(255) NOT NULL' );
					$this->append_log( array( 'message' => 'CDP Message: Inventory Import DB Updated added column vehicle_url, ' . $this->return_result( $result ) ) );
					$result = self::$wpdb->query( 'ALTER TABLE cdp_inventory_' . self::$blog . ' ADD COLUMN vehicle_url varchar(255) NOT NULL' );
					$this->append_log( array( 'message' => 'CDP Message: Inventory DB Updated added column vehicle_url, ' . $this->return_result( $result ) ) );
				}

			}
			$this->admin_options['admin']['database']['inventory']['current'] = $this->default_current_inventory_version;
		}


	public function return_result( $result ) {
		return ' result: ' . $result ? 'true' : 'false';
	}
}
