<?php

namespace CarDealerPress\Includes\Database;

use CarDealerPress\includes\functions\Logger;
use CarDealerPress\includes\helpers\helpers as helpers;
use CarDealerPress\Includes\Helpers\Remote_Requests;
use CarDealerPress\Includes\Helpers\Transients;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class Inserts {
	use Logger;

	private static $blog;
	private static $wpdb;
	/**
	 * @var int|mixed
	 */
	public $company_id = 0;

	public function __construct() {
		global $wpdb;
		self::$wpdb = $wpdb;
		self::$blog = get_current_blog_id();
	}

	public function cdp_data_and_cdp_companies( $dealer_id = 0 ) {
		if ( $dealer_id ) {
			$this->company_id     = $dealer_id;
			$query_prepare        = 'select * from cdp_data where blog_id = %d and data_key = %s and dealer_id = %d';
			$query_params_prepare = array( self::$blog, 'dealer', $dealer_id );
			$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
			$data_dealer          = self::$wpdb->get_results( $query, ARRAY_A );
			// This should be empty so duplicates are not added.
			if ( empty( $data_dealer ) ) {
				$api_params    = '/api/companies/' . $dealer_id;
				$response      = Remote_Requests::http_get_file( $api_params );
				$response_code = wp_remote_retrieve_response_code( $response );
				if ( 200 === $response_code ) {
					$response_body = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
					$this->add_single_dealer_to_cdp_data_table( $dealer_id );
					$this->populate_companies_table(
						array(
							'data'       => $response_body,
							'company_id' => $dealer_id,
						)
					);
					if ( $response_body->automall_ids ?? false ) {
						$this->add_automalls_dealer_to_cdp_table( $response_body->automall_ids );
						foreach ( $response_body->automall_ids as $dealer_id ) {
							$response_body = $this->call_api_company_for( $dealer_id );
							$this->populate_companies_table(
								array(
									'data'       => $response_body,
									'company_id' => helpers::company_id(),
								)
							);
						}
					}
					Transients::delete_cdp_company_processing( self::$blog );
				}
			}
		}
	}

	public function cdp_data( $new_dealer = 0 ) {
		if ( $new_dealer ) {
			$query_prepare        = 'select * from cdp_data where blog_id = %d and data_key = %s and dealer_id = %d';
			$query_params_prepare = array( self::$blog, 'dealer', $new_dealer );
			$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
			$data_dealer          = self::$wpdb->get_results( $query, ARRAY_A );
			// This should be empty so duplicates are not added.
			if ( empty( $data_dealer ) ) {
				$api_params    = '/api/companies/' . $new_dealer;
				$response      = Remote_Requests::http_get_file( $api_params );
				$response_code = wp_remote_retrieve_response_code( $response );
				if ( 200 === $response_code ) {
					$response_body = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
					$this->add_single_dealer_to_cdp_data_table( $new_dealer );
					if ( $response_body->automall_ids ?? false ) {
						$this->add_automalls_dealer_to_cdp_table( $response_body->automall_ids );
					}
					Transients::delete_cdp_company_processing( self::$blog );
				}
			}
		}
	}

	public function cdp_check_for_valid_company_id_and_populate_companies_table( $new_dealer = 0 ) {
		if ( 0 !== $new_dealer ) {
			$api_params    = '/api/companies/' . $new_dealer;
			$response      = Remote_Requests::http_get_file( $api_params );
			$response_code = wp_remote_retrieve_response_code( $response );
			if ( 200 === $response_code ) {
				$response_body = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
				$this->populate_companies_table(
					array(
						'data'       => $response_body,
						'company_id' => $new_dealer,
					)
				);
				if ( $response_body->automall_ids ?? false ) {
					foreach ( $response_body->automall_ids as $dealer ) {
						$response_body = $this->call_api_company_for( $dealer );
						$this->populate_companies_table(
							array(
								'data'       => $response_body,
								'company_id' => $dealer,
							)
						);
					}
				}
				Transients::delete_cdp_company_processing( self::$blog );
			}
		}
	}

	public function call_api_company_for( $dealer ) {
		// TODO: future possible move to another class, maybe remote_requests.
		$api_params    = '/api/companies/' . $dealer;
		$response      = Remote_Requests::http_get_file( $api_params );
		$response_code = wp_remote_retrieve_response_code( $response );
		if ( 200 === $response_code ) {
			return isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
		}

	}

	public function add_automalls_dealer_to_cdp_table( $dealers ) {
		foreach ( $dealers as $dealer ) {
			$this->append_log( array( 'message' => 'CDP Message: Automall Dealer (' . $dealer . ') added to CDP Data table' ) );
			$data   = array(
				'blog_id'    => self::$blog,
				'dealer_id'  => $dealer,
				'data_key'   => 'dealer',
				'data_value' => 'automall',
			);
			$format = array( '%d', '%d', '%s', '%s' );
			self::$wpdb->insert( 'cdp_data', $data, $format );
		}
	}

	public function add_single_dealer_to_cdp_data_table( $dealer ) {
		$data   = array(
			'blog_id'    => self::$blog,
			'dealer_id'  => $dealer,
			'data_key'   => 'dealer',
			'data_value' => $dealer,
		);
		$format = array( '%d', '%d', '%s', '%s' );
		self::$wpdb->insert( 'cdp_data', $data, $format );
	}

	public function populate_companies_table( $args ) {
		$response_body = $args['data'] ?? '';
		$type          = $args['type'] ?? 'insert';
		$company_id    = $args['company_id'] ?? helpers::company_id();

		if ( $response_body ) {
			$table_data = array();

			$dealer_id    = $response_body->id ?? 0;
			$name         = $response_body->name ?? '';
			$street       = $response_body->street ?? '';
			$city         = $response_body->city ?? '';
			$state        = $response_body->state ?? '';
			$zip          = $response_body->zip ?? '';
			$phone        = $response_body->phone ?? '';
			$country_code = $response_body->country_code ?? '';
			$automalls    = $response_body->automall_ids ?? '';
			$created      = ( isset( $response_body->created ) ? wp_date( 'Y-m-d', strtotime( $response_body->created ) ) : '2017-04-01' );
			$updated      = ( isset( $response_body->updated ) ? strtotime( $response_body->updated ) : time() );
			$seo_response = $response_body->seo ?? '';
			$seo          = serialize( json_decode( wp_json_encode( $seo_response ), true ) );
			$runtime      = 0;

			$table_data[] = array(
				'blog_id'                     => self::$blog,
				'company_id'                  => $company_id,
				'dealer_id'                   => $dealer_id,
				'name'                        => $name,
				'street'                      => $street,
				'city'                        => $city,
				'state'                       => $state,
				'zip'                         => $zip,
				'phone'                       => $phone,
				'country_code'                => $country_code,
				'automalls'                   => serialize( $automalls ),
				'created'                     => $created,
				'last_updated'                => $updated,
				'inventory_last_updated'      => $runtime,
				'inventory_api_last_modified' => 0,
				'seo'                         => $seo,
			);

			if ( $table_data && 'insert' === $type ) {
				self::$wpdb->insert( 'cdp_companies', $table_data[0] );
			}
			if ( $table_data && 'update' === $type ) {
				$where = array( 'dealer_id' => $dealer_id );
				self::$wpdb->update( 'cdp_companies', $table_data[0], $where );
			}
		}
	}

}
