<?php

namespace CarDealerPress\Includes\Database;

use CarDealerPress\Includes\Functions\Logger;
use CarDealerPress\includes\helpers\helpers;
use CarDealerPress\Includes\Helpers\{Generic, Option, Remote_Requests};

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Inventory_Handler {
	use Logger;
	private $purge;
	private static $wpdb;
	private static $blog;
	public $page_count;
	public $run_purge;
	public $table_data;
	public $sort_data;
	public $dealer;
	public $company_id;
	public $theme_options;
	public $admin_options;
	private int $total_count;
	private array $vehicle_data;

	public function __construct() {
		global $wpdb;
		self::$wpdb          = $wpdb;
		self::$blog          = get_current_blog_id();
		$this->theme_options = Option::get_cdp_theme_options();
		$this->admin_options = Option::get_cdp_admin_options();
	}

	public function init_inventory_handler( $dealer_id = 0, $purge = false ) {
		$single_dealer = '';
		if ( 0 !== $dealer_id ) {
			$single_dealer = ' AND dealer_id = %d';
		}
		if ( true === $purge ) {
			$this->run_purge = true;
		}
		$query_prepare = 'select * from cdp_companies where blog_id = %d' . $single_dealer;
		if ( empty( $single_dealer ) ) {
			$query_params_prepare = array( self::$blog );
		} else {
			$query_params_prepare = array( self::$blog, $dealer_id );
		}
		$query   = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
		$dealers = self::$wpdb->get_results( $query, ARRAY_A );
		foreach ( $dealers as $key => $dealer ) {
			$automall_check = unserialize( $dealer['automalls'] );
			if ( empty( $automall_check ) ) {
				// Allow the process to run for 2 minutes needed for large automalls.
				set_time_limit( 120 );
				$this->dealer     = $dealer;

				$this->company_id = $dealer['company_id'];

				$api_params       = Generic::api_vehicles_params( array( 'dealer_id' => $dealer['dealer_id'] ) );
				$response         = Remote_Requests::http_get_file( $api_params );
				$response_code    = wp_remote_retrieve_response_code( $response );
				// Confirm response_code returns 200 before removing inventory and re updating.
				if ( 200 === $response_code ) {
					// Set Last Modified value from API.
					$last_modified = isset( $response['headers']['last-modified'] ) ? strtotime( $response['headers']['last-modified'] ) : 0;
					// Clear Import Table.
					( new Truncate() )->clear_import_table();
					// Create page count variable.
					$response_data     = isset( $response['body'] ) ? json_decode( $response['body'] ) : array();
					$this->total_count = is_array( $response_data ) && isset( $response_data[0] ) && count( $response_data ) > 0 ? $response_data[0]->pagination->total : 0;
					$this->page_count  = ( $this->total_count > 0 ? $this->total_count + 1 : 0 );
					// Populate Import Table.
					$result = $this->inventory_call_data();
					if ( false !== $result ) {
						// Remove Current Dealer Vehicles from Live Inventory Update.
						self::$wpdb->delete(
							'cdp_inventory_' . self::$blog,
							array(
								'blog_id'   => self::$blog,
								'dealer_id' => $dealer['dealer_id'],
							),
							array( '%d', '%d' )
						);
						// Populate Live Table.
						$this->update_live_table();
						// Update Company Import Times.
						$update_values = array(
							'inventory_last_updated'      => time(),
							'inventory_api_last_modified' => $last_modified,
						);
						$where_values  = array(
							'blog_id'   => self::$blog,
							'dealer_id' => $dealer['dealer_id'],
						);
						$format        = array( '%d', '%d' );
						self::$wpdb->update( 'cdp_companies', $update_values, $where_values, $format, $format );
						// Update Sort Values.
						if ( $this->sort_data ) {
							( new Requests() )->cdp_admin_update_sort( $this->sort_data );
						}
					} else {
						$this->append_log( array( 'message' => 'CDP Message: Inventory Update Handler failed, API response exceeded timeout condition with WP call, or cron was terminated by server.' ) );
					}
				} elseif ( 403 === $response_code ) {
					$this->append_log( array( 'message' => 'CDP Message: Dealer id: ' . $dealer['dealer_id'] . ' Suspended, removing inventory' ) );
					$where        = array(
						'blog_id'    => self::$blog,
						'company_id' => $this->company_id,
						'dealer_id'  => $dealer['dealer_id'],
					);
					$where_format = array( '%d', '%d', '%d' );
					self::$wpdb->delete( 'cdp_inventory_' . self::$blog, $where, $where_format );
				} else {
					$this->append_log( array( 'message' => 'CDP Message: Inventory Update Handler check failed | response code ' . $response_code ) );
				}
			} else {
				$this->append_log( array( 'message' => 'CDP Message: Manual Update has dealer ' . $dealer['dealer_id'] . ' as a main automall account. Will not update.' ) );
			}
		}
	}

	public function inventory_call_data() {
		$this->vehicle_data = $vehicle_array = array();
		$param_list         = $this->build_inventory_request_list();
		foreach ( $param_list as $key => $api_params ) {
			if ( $this->run_purge ) {
				Remote_Requests::purge_url( $api_params );
			}
			$response      = Remote_Requests::http_get_file( $api_params );
			$response_code = wp_remote_retrieve_response_code( $response );
			$this->append_log( array( 'message' => 'CDP Message: CDP API Import Page | code: ' . $response_code . ' | request: ' . $api_params ) );
			// First Try.
			if ( 200 === $response_code ) {
				$this->scrub_build_populate( $api_params, $response );
			} else { // Second try.
				sleep( 2 );
				$response      = Remote_Requests::http_get_file( $api_params );
				$response_code = wp_remote_retrieve_response_code( $response );
				$this->append_log( array( 'message' => 'CDP Message: CDP API Import Page 2nd Attempt | code: ' . $response_code . ' | request: ' . $api_params ) );
				if ( 200 === $response_code ) {
					$this->scrub_build_populate( $api_params, $response );
				} else {
					$this->append_log( array( 'message' => 'CDP Message: CDP API Import Page 2nd Attempt failed | request: ' . $api_params ) );
					$this->append_log( array( 'message' => 'CDP Message: CDP API Import | Stopping Import' ) );

					// Exit out of loop if second call fails, and not import data.
					return false;
					exit;
				}
			}
		}
	}

	public function scrub_build_populate( $api_params, $response ) {
		$response_data = isset( $response['body'] ) ? json_decode( $response['body'] ) : array();
		if ( ! empty( $response_data ) ) {
			$vehicles             = $this->scrub_vehicle( $response_data );
			$this->vehicle_data[] = $vehicles;
			$this->build_inventory_table_data();
			unset( $this->vehicle_data );
			$this->populate_import_table();
		} else {
			$this->append_log( array( 'message' => 'CDP Message: Response Body Empty for params: ' . $api_params ) );
		}
	}

	public function build_inventory_request_list() {
		$param_url_list = array();
		$i              = 0;
		$page           = 1;
		do {
			if ( $page <= $this->page_count ) {
				$params           = Generic::api_vehicles_params(
					array(
						'dealer_id' => $this->dealer['dealer_id'],
						'page'      => $page,
					)
				);
				$param_url_list[] = $params;
				$page ++;
				$i ++;
			} else {
				break 1;
			}
		} while ( $i < $this->page_count );
		$this->append_log( array( 'message' => 'CDP Message: Import build_inventory_request_list full list: ' . wp_json_encode( $param_url_list ) ) );

		return $param_url_list;
	}

	public function scrub_vehicle( $data ) {
		$result = array();
		$keys   = array(
			'stock_number',
			'vin',
			'saleclass',
			'make',
			'model_name',
			'trim',
			'year',
			'contact_info',
			'exterior_color',
			'interior_color',
			'tags',
			'body_style',
			'vehicleclass',
			'id',
			'drive_train',
			'prices',
			'video_url',
			'doors',
			'engine',
			'transmission',
			'odometer',
			'carfax',
			'carfax_image_url',
			'photos',
			'headline',
			'description',
			'certified_bool',
			'created_at',
			'ads_acode',
			'updated_at',
			'sold_on',
			'auto_check_url',
			'vehicle_location',
			'fuel_type',
			'model_code',
			'fuel_economy',
			'vehicle_url',
		);
		if ( $data ) {
			foreach ( $data as $i => $values ) {
				foreach ( $keys as $key ) {
					if ( is_array( $values ) ) {
						$result[ $i ][ $key ] = ( $values[ $key ] ?? '' );
					} else {
						$result[ $i ][ $key ] = ( $values->$key ?? '' );
					}
				}
			}
		}

		return $result;
	}

	public function build_inventory_table_data() {
		$this->table_data = array();
		if ( ! empty( $this->vehicle_data ) ) {
			foreach ( $this->vehicle_data as $page => $list ) {
				foreach ( $list as $vehicle ) {
					$id           = ( $vehicle['id'] ?? 0 );
					$stock        = ( $vehicle['stock_number'] ?? '' );
					$vin          = ( $vehicle['vin'] ?? '' );
					$saleclass    = ( isset( $vehicle['saleclass'] ) ? strtolower( $vehicle['saleclass'] ) : '' );
					$make         = ( $vehicle['make'] ?? '' );
					$model        = ( $vehicle['model_name'] ?? '' );
					$model_code   = ( $vehicle['model_code'] ?? '' );
					$trim         = ( $vehicle['trim'] ?? '' );
					$year         = ( $vehicle['year'] ?? '' );
					$exterior     = ( $vehicle['exterior_color'] ?? '' );
					$interior     = ( $vehicle['interior_color'] ?? '' );
					$color_map    = '';
					$body         = ( $vehicle['body_style'] ?? '' );
					$vehicleclass = ( $vehicle['vehicleclass'] ?? '' );
					$tags         = ( ! empty( $vehicle['tags'] ) ? implode( ',', $vehicle['tags'] ) : '' );
					if ( is_array( $vehicle['contact_info'] ) ) {
						$dealer = ( $vehicle['contact_info']['company_id'] ?? '' );
					} else {
						$dealer = ( $vehicle['contact_info']->company_id ?? '' );
					}

					$prices       = ( $vehicle['prices'] ?? array() );
					$drivetrain   = ( $vehicle['drive_train'] ?? '' );
					$video        = ( $vehicle['video_url'] ?? '' );
					$doors        = ( $vehicle['doors'] ?? '' );
					$engine       = ( $vehicle['engine'] ?? '' );
					$transmission = ( $vehicle['transmission'] ?? '' );
					$fuel         = ( isset( $vehicle['fuel_type'] ) ? ( 'N/A' == $vehicle['fuel_type'] ? '' : $vehicle['fuel_type'] ) : '' );
					$fuel_economy = ( $vehicle['fuel_economy'] ?? array() );
					$odometer     = ( $vehicle['odometer'] ?? '' );
					if ( is_array( $vehicle['carfax'] ) ) {
						$carfax           = ( $vehicle['carfax']['url'] ?? '' );
						$carfax_image_url = ( $vehicle['carfax']['image_url'] ?? '' );
					} else {
						$carfax           = ( $vehicle['carfax']->url ?? '' );
						$carfax_image_url = ( $vehicle['carfax']->image_url ?? '' );
					}
					$autocheck   = ( $vehicle['auto_check_url'] ?? '' );
					$photos      = ( $vehicle['photos'] ?? array() );
					$headline    = ( isset( $vehicle['headline'] ) ? substr( $vehicle['headline'], 0, 512 ) : '' );
					$description = ( $vehicle['description'] ?? '' );
					$certified   = ( isset( $vehicle['certified_bool'] ) && true == $vehicle['certified_bool'] ? 1 : 0 );
					$created     = ( $vehicle['created_at'] ?? '' );
					$updated     = ( $vehicle['updated_at'] ?? '' );
					$sold_on     = ( $vehicle['sold_on'] ?? null );
					$acode       = ( $vehicle['ads_acode'] ?? '' );
					$location    = ( $vehicle['vehicle_location'] ?? '' );
					$blog_id     = get_current_blog_id();
					// Find sortprice lowest value.
					if ( is_array( $prices ) ) {
						if ( isset( $prices['sale_price'] ) ) {
							$sortprice = ( 0 !== $prices['sale_price'] && $prices['sale_price'] < $prices['asking_price'] ? $prices['sale_price'] : $prices['asking_price'] );
						} elseif ( isset( $prices['asking_price'] ) ) {
							$sortprice = $prices['asking_price'];
						} else {
							$sortprice = 0;
						}
					} else {
						if ( isset( $prices->sale_price ) ) {
							$sortprice = ( 0 !== $prices->sale_price && $prices->sale_price < $prices->asking_price ? $prices->sale_price : $prices->asking_price );
						} elseif ( isset( $prices->asking_price ) ) {
							$sortprice = $prices->asking_pricek;
						} else {
							$sortprice = 0;
						}
					}

					// Edge Case.
					$extra  = ( 'sport utility' === strtolower( $vehicleclass ) ? 'SUV ' : '' );
					$extra .= ( 'chevrolet' === strtolower( $make ) ? 'Chevy ' : '' );
					$extra .= ( 'f-150' === strtolower( $model ) ? 'F150 ' : '' );

					$vehicle_url = ( $vehicle['vehicle_url'] ?? '' );

					if ( is_array( $prices ) ) {
						$askingprice = ( $prices['asking_price'] ?? 0 );
						$saleprice   = ( $prices['sale_price'] ?? 0 );
						$retailprice = ( $prices['retail_price'] ?? 0 );
						$msrp        = ( $prices['msrp'] ?? 0 );
						$sale_expire = ( $prices['sale_expire'] ?? 0 );
					} else {
						$askingprice = ( $prices->asking_price ?? 0 );
						$saleprice   = ( $prices->sale_price ?? 0 );
						$retailprice = ( $prices->retail_price ?? 0 );
						$msrp        = ( $prices->msrp ?? 0 );
						$sale_expire = ( $prices->sale_expire ?? 0 );
					}
					if ( is_array( $fuel_economy ) ) {
						$highway = ( $fuel_economy['highway'] ?? 0 );
						$city    = ( $fuel_economy['city'] ?? 0 );
					} else {
						$highway = ( $fuel_economy->highway ?? 0 );
						$city    = ( $fuel_economy->city ?? 0 );
					}
					if ( is_array( $photos[0] ) ) {
						$thumbnail = ( $photos[0]['small'] ?? '' );
					} else {
						$thumbnail = ( $photos[0]->small ?? '' );
					}

					$this->table_data[] = array(
						'blog_id'          => $blog_id,
						'vms_id'           => $id,
						'company_id'       => $this->company_id,
						'dealer_id'        => $dealer,
						'dealer'           => ( $this->dealer['name'] ?? '' ),
						'stock'            => $stock,
						'vin'              => $vin,
						'saleclass'        => $saleclass,
						'certified'        => $certified,
						'make'             => $make,
						'model'            => $model,
						'model_code'       => $model_code,
						'trim'             => $trim,
						'year'             => $year,
						'askingprice'      => $askingprice,
						'saleprice'        => $saleprice,
						'retailprice'      => $retailprice,
						'msrp'             => $msrp,
						'sortprice'        => $sortprice,
						'pricesort'        => 0,
						'sale_expire'      => $sale_expire,
						'headline'         => $headline,
						'description'      => $description,
						'exterior'         => $exterior,
						'interior'         => $interior,
						'color_map'        => '', // Gets set on own update.
						'body'             => $body,
						'vehicleclass'     => $vehicleclass,
						'tags'             => ( empty( $tags ) ? '' : $tags ),
						'drivetrain'       => $drivetrain,
						'doors'            => $doors,
						'engine'           => $engine,
						'transmission'     => $transmission,
						'fuel_type'        => $fuel,
						'highway'          => $highway,
						'city'             => $city,
						'odometer'         => $odometer,
						'carfax'           => $carfax,
						'carfax_image_url' => $carfax_image_url,
						'autocheck'        => $autocheck,
						'video'            => $video,
						'acode'            => $acode,
						'thumbnail'        => $thumbnail,
						'geo_state'        => ( $this->dealer['state'] ?? '' ),
						'geo_city'         => ( $this->dealer['city'] ?? '' ),
						'geo_zip'          => ( $this->dealer['zip'] ?? '' ),
						'location'         => $location,
						'phone'            => ( $this->dealer['phone'] ?? '' ),
						'created'          => $created,
						'updated'          => $updated,
						'sold_on'          => $sold_on,
						'sort'             => 0,
						'extra'            => $extra,
						'vehicle_url'      => $vehicle_url,
					);
				}
			}
		}
	}

	public function populate_import_table() {

		if ( $this->table_data ) {
			$count = count( $this->table_data );
			$i     = 0;
			do {
				self::$wpdb->insert( 'cdp_inventory_import_' . self::$blog . '', $this->table_data[ $i ] );
				$i ++;
			} while ( $i < $count );
			unset( $this->table_data );
		}
	}

	public function update_live_table() {
		$this->run_custom_imports();
		// Removes Possible Duplicates.
		$sql = 'DELETE FROM cdp_inventory_' . self::$blog . ' WHERE id in ( SELECT max_id from ( SELECT max(id) as max_id FROM cdp_inventory_' . self::$blog . ' WHERE vms_id IN ( SELECT vms_id FROM (SELECT vms_id, count(*) AS duplicates FROM cdp_inventory_' . self::$blog . ' WHERE company_id = ' . $this->company_id . ' AND blog_id = ' . self::$blog . ' AND dealer_id = ' . $this->dealer['dealer_id'] . ' GROUP BY vms_id ) AS z where z.duplicates > 1 ) GROUP BY vms_id ) AS a )';
		self::$wpdb->query( $sql );
		// Update Color.
		if ( 'on' === $this->theme_options['theme']['inventory_display']['list']['search_filters']['color_mapping']['enable'] ) {
			$this->init_inventory_color_update();
		}
		// Update PriceSort
		$sql = 'UPDATE cdp_inventory_' . self::$blog . ' SET pricesort = 99 WHERE sortprice = 0 AND company_id = ' . $this->company_id . ' AND blog_id = ' . self::$blog . ' AND dealer_id = ' . $this->dealer['dealer_id'];
		self::$wpdb->query( $sql );
		// Reset PriceSort.
		$sql = 'UPDATE cdp_inventory_' . self::$blog . ' SET pricesort = 0 WHERE pricesort = 99 and sortprice <> 0 AND company_id = ' . $this->company_id . ' AND blog_id = ' . self::$blog . ' AND dealer_id = ' . $this->dealer['dealer_id'];
		self::$wpdb->query( $sql );
	}

	public function run_custom_imports() {
		$skip_custom_import = false;
		// Get list of Dealers.
		$query     = 'SELECT dealer_id FROM cdp_inventory_import_' . self::$blog . ' WHERE blog_id = ' . self::$blog . ' group by dealer_id';
		$import_id = self::$wpdb->get_results( $query, ARRAY_A );
		if ( 1 === count( $import_id ) && ! empty( $import_id[0]['dealer_id'] ) ) {
			// Delete Sold Vehicles | Custom Not Needed.
			$query  = 'SELECT ID FROM cdp_inventory_' . self::$blog . ' WHERE cdp_inventory_' . self::$blog . '.company_id = ' . $this->company_id . ' AND cdp_inventory_' . self::$blog . '.blog_id = ' . self::$blog . ' AND cdp_inventory_' . self::$blog . '.dealer_id = ' . $this->dealer['dealer_id'] . ' AND cdp_inventory_' . self::$blog . '.vms_id NOT IN( SELECT a.vms_id FROM cdp_inventory_import_' . self::$blog . ' as a WHERE cdp_inventory_' . self::$blog . '.company_id = a.company_id AND a.blog_id = ' . self::$blog . ' AND cdp_inventory_' . self::$blog . '.dealer_id = a.dealer_id GROUP BY a.vms_id)';
			$remove = self::$wpdb->get_results( $query, ARRAY_A );
			if ( $remove ) {
				foreach ( $remove as $row ) {
					$table        = 'cdp_inventory_' . self::$blog;
					$where        = array( 'id' => $row['ID'] );
					$where_format = array( '%d' );
					self::$wpdb->delete( $table, $where, $where_format );
				}
			}
			// Removes Out Dated Vehicles, meaning they need to be updated with new data.
			$query  = 'SELECT ID FROM cdp_inventory_' . self::$blog . ' WHERE cdp_inventory_' . self::$blog . '.company_id = ' . $this->company_id . ' AND cdp_inventory_' . self::$blog . '.blog_id = ' . self::$blog . ' AND cdp_inventory_' . self::$blog . '.dealer_id = ' . $this->dealer['dealer_id'] . ' AND cdp_inventory_' . self::$blog . '.vms_id IN( SELECT a.vms_id FROM (SELECT vms_id, updated FROM cdp_inventory_' . self::$blog . ' WHERE company_id = ' . $this->company_id . ' AND blog_id = ' . self::$blog . ' AND dealer_id = ' . $this->dealer['dealer_id'] . ') as a JOIN cdp_inventory_import_' . self::$blog . ' b ON a.vms_id = b.vms_id WHERE a.updated <> b.updated GROUP BY a.vms_id)';
			$remove = self::$wpdb->get_results( $query, ARRAY_A );
			if ( $remove ) {
				foreach ( $remove as $row ) {
					$table        = 'cdp_inventory_' . self::$blog;
					$where        = array( 'id' => $row['ID'] );
					$where_format = array( '%d' );
					self::$wpdb->delete( $table, $where, $where_format );
				}
			}

			// Get custom imports.
			$import_filters = $this->admin_options['admin']['advanced']['import_filters'];
			$import_filter  = array();

			if ( empty( $import_filters ) ) {
				$skip_custom_import = true;
			} else {
				foreach ( $import_filters as $key => $filter ) {
					if ( $filter['dealer_id'] === $import_id[0]['dealer_id'] ) {
						$import_filter = $filter;
					}
				}
			}

			$insert_columns  = 'blog_id, vms_id, company_id, dealer_id, dealer, stock, vin, saleclass, certified, make,';
			$insert_columns .= ' model, model_code, trim, year, askingprice, saleprice, retailprice, msrp, sortprice, pricesort, sale_expire,';
			$insert_columns .= ' headline, description, exterior, interior, color_map, body, vehicleclass, tags, drivetrain, doors, engine,';
			$insert_columns .= ' transmission, fuel_type, highway, city, odometer, carfax, carfax_image_url, autocheck, video, acode, thumbnail,';
			$insert_columns .= ' geo_state, geo_city, geo_zip, location, phone, created, updated, sold_on, sort, extra, vehicle_url';

			$custom_query = $skip_custom_import ? '' : $this->build_custom_import_query( $import_filter );
			$query        = 'SELECT ' . $insert_columns . ' FROM cdp_inventory_import_' . self::$blog;
			$query       .= ' as z WHERE z.vms_id NOT IN( SELECT a.vms_id FROM cdp_inventory_' . self::$blog;
			$query       .= ' as a WHERE z.company_id = a.company_id and z.dealer_id = a.dealer_id AND a.blog_id = ' . self::$blog;
			$query       .= ' ' . $custom_query . ' GROUP BY a.vms_id ) ' . $custom_query;
			$new_rows     = self::$wpdb->get_results( $query, ARRAY_A );
			if ( $new_rows ) {
				$count = count( $new_rows );
				$i     = 0;
				do {
					self::$wpdb->insert( 'cdp_inventory_' . self::$blog . '', $new_rows[ $i ] );
					$i ++;
				} while ( $i < $count );
				unset( $new_rows );
			}
		}
	}

	public function init_inventory_color_update() {
		$vehicles = ( new Requests() )->cdp_inventory_special_request( 'id,exterior' );
		if ( $vehicles ) {
			foreach ( $vehicles as $key => $values ) {
				$color   = $this->exterior_color_mapping( $values['exterior'] );
				$updated = self::$wpdb->update( 'cdp_inventory_' . self::$blog, array( 'color_map' => $color ), array( 'id' => $values['id'] ), array( '%s' ), array( '%d' ) );
				if ( false === $updated ) {
					$this->append_log(
						array(
							'message' => 'CDP Message: CDP Color Map error on ID ' . $values['id'],
							'type'    => 'ERROR',
						)
					);
				}
			}
		}
		// Copies logic for Exterior Color Map but for Interior.
		$vehicles = ( new Requests() )->cdp_inventory_special_request( 'id,interior' );
		if ( $vehicles ) {
			foreach ( $vehicles as $key => $values ) {
				$color   = $this->exterior_color_mapping( $values['interior'] );
				$updated = self::$wpdb->update( 'cdp_inventory_' . self::$blog, array( 'color_map_int' => $color ), array( 'id' => $values['id'] ), array( '%s' ), array( '%d' ) );
				if ( false === $updated ) {
					$this->append_log(
						array(
							'message' => 'CDP Message: CDP Color Map error on ID ' . $values['id'],
							'type'    => 'ERROR',
						)
					);
				}
			}
		}
	}

	public function exterior_color_mapping( $color ) {
		$default_color = ( $this->theme_options['theme']['inventory_display']['list']['search_filters']['color_mapping']['default_text'] ?? 'Unknown' );
		if ( empty( $color ) ) {
			return $default_color;
		} else {
			$color_map = '';
			$color     = strtolower( $color );
			foreach ( $this->theme_options['theme']['inventory_display']['list']['search_filters']['color_mapping']['exterior'] as $key => $values ) {
				if ( empty( $color_map ) ) {
					$color_search = explode( ',', $values );
					foreach ( $color_search as $needle ) {
						$needle = trim( $needle );
						$pos    = strpos( $color, strtolower( $needle ) );
						if ( false !== $pos ) {
							$color_map = ucfirst( $key );
							break;
						}
					}
				} else {
					break;
				}
			}
			if ( empty( $color_map ) ) {
				return $default_color;
			} else {
				return $color_map;
			}
		}
	}

	public function build_custom_import_query( $import ) {
		$custom_string = '';
		$search        = '';
		$clean_data    = array();
		if ( ! empty( $import ) ) {
			$custom = $import;
			// Build custom saleclass.
			if ( ! ( 'all' === $custom['saleclass'] ) ) {
				if ( 'certified' === $custom['saleclass'] ) {
					$custom_string .= ' AND saleclass = "used" AND certified = 1';
				} else {
					$custom_string .= ' AND saleclass = "' . $custom['saleclass'] . '"';
				}
			}
			// Build custom make.
			if ( ! empty( $custom['makes'] ) ) {
				$data   = explode( ',', $custom['makes'] );
				$symbol = $this->get_custom_import_symbol( $custom['make_qualifier'] );
				// Clean white space.
				foreach ( $data as $value ) {
					$clean_data[] = trim( $value );
				}
				$search         = ' AND make ' . $symbol . ' ("' . implode( '","', $clean_data ) . '") ';
				$custom_string .= $search;
			}
			// Build custom year.
			if ( ! empty( $custom['years'] ) ) {
				$data   = explode( ',', $custom['years'] );
				$symbol = $this->get_custom_import_symbol( $custom['year_qualifier'] );
				// Clean white space.
				foreach ( $data as $value ) {
					$clean_data[] = trim( $value );
				}
				if ( in_array( $custom['year_qualifier'], array( 2, 3 ), true ) ) {
					$search = ' AND year ' . $symbol . '"' . $clean_data[0] . '" ';
				} else {
					$search = ' AND year ' . $symbol . ' ("' . implode( '","', $clean_data ) . '") ';
				}
				$custom_string .= $search;
			}
		}

		return $custom_string;
	}

	public function get_custom_import_symbol( $key ) {
		$result = '';
		switch ( strtolower( $key ) ) {
			case 'equal':
				$result = ' IN ';
				break;
			case 'not equal':
				$result = ' NOT IN ';
				break;
			case 'greater than':
				$result = ' > ';
				break;
			case 'less than':
				$result = ' < ';
				break;
		}

		return $result;
	}

}
