<?php

namespace CarDealerPress\Includes\Database;

use CarDealerPress\includes\functions\Logger;
use CarDealerPress\Includes\Helpers\Option;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Update_Delete {
	use Logger;

	public static $admin_options;
	public static $theme_options;

	private static $wpdb;
	private static $blog;

	public function __construct() {
		global $wpdb;
		self::$wpdb          = $wpdb;
		self::$theme_options = Option::get_cdp_theme_options();
		self::$admin_options = Option::get_cdp_admin_options();
		self::$blog          = get_current_blog_id();
	}

	public function remove_dealer_id_from_cdp_tables( $dealer_id ) {
		$this->remove_dealer_id_from_cdp_data_table( $dealer_id );
		$this->remove_dealer_id_from_cdp_companies_table( $dealer_id );
		$this->remove_dealer_id_from_table(
			array(
				'table'     => 'cdp_inventory_',
				'dealer_id' => $dealer_id,
			)
		);
		$this->remove_dealer_id_from_table(
			array(
				'table'     => 'cdp_inventory_import_',
				'dealer_id' => $dealer_id,
			)
		);
	}

	public function remove_dealer_id_from_cdp_data_table( $dealer_id ) {
		// Remove Data from CDP Data.
		$this->append_log( array( 'message' => 'CDP Message: Removing dealer ' . $dealer_id . ' from CDP Data Table' ) );
		$where        = array(
			'blog_id'   => self::$blog,
			'data_key'  => 'dealer',
			'dealer_id' => $dealer_id,
		);
		$where_format = array( '%d', '%s', '%d' );
		self::$wpdb->delete( 'cdp_data', $where, $where_format );
		$where        = array(
			'blog_id'  => self::$blog,
			'data_key' => 'disclaimer',
		);
		$where_format = array( '%d', '%s' );
		self::$wpdb->delete( 'cdp_data', $where, $where_format );
	}

	public function remove_dealer_id_from_cdp_companies_table( $dealer_id ) {
		// Remove Data from CDP Data.
		$this->append_log( array( 'message' => 'CDP Message: Removing dealer ' . $dealer_id . ' from CDP Companies Table' ) );
		$where        = array(
			'blog_id'   => self::$blog,
			'dealer_id' => $dealer_id,
		);
		$where_format = array( '%d', '%s', '%d' );
		self::$wpdb->delete( 'cdp_companies', $where, $where_format );
	}

	public function remove_dealer_id_from_table( $args ) {
		$this->append_log( array( 'message' => 'CDP Message: Removing dealer ' . $args['dealer_id'] . ' from Table ' . $args['table'] . ' for blog ' . self::$blog ) );
		$where        = array(
			'blog_id'   => self::$blog,
			'dealer_id' => $args['dealer_id'],
		);
		$where_format = array( '%d', '%d' );
		self::$wpdb->delete( $args['table'] . self::$blog, $where, $where_format );
	}

	public function reset_tables( $dealer_id ) {
		$this->append_log( array( 'message' => 'CDP Message: Dealer Id has been changed or a reset has been triggered, truncating all data tables from blog site:' . self::$blog ) );
		$this->remove_dealer_id_from_cdp_data_table( $dealer_id );
		$this->remove_dealer_id_from_cdp_companies_table( $dealer_id );
		self::$wpdb->query( 'TRUNCATE cdp_inventory_' . self::$blog );
		self::$wpdb->query( 'TRUNCATE cdp_inventory_import_' . self::$blog );
	}

	public function remove_all_cdp_tables() {
			// Remove Data from CDP Data.
			$tables = array(
				'cdp_data',
				'cdp_companies',
				'cdp_inventory_import_' . self::$blog,
				'cdp_inventory_' . self::$blog,
			);
			foreach ( $tables as $table ) {
				// @codingStandardsIgnoreStart
				$this->append_log( array( 'message' => 'CDP Message: Removing table ' . $table ) );
				$query                = self::$wpdb->query( "DROP TABLE IF EXISTS $table" );
				// @codingStandardsIgnoreEnd
			}
	}

}
