<?php

namespace CarDealerPress\Includes\Functions;

use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\Includes\Helpers\Remote_Requests;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Sc_Ajax {
	private $options;
	private $vms;
	private $company;

	public function __construct( $options, $vms, $company ) {
		$this->options = $options;
		$this->vms     = $vms;
		$this->company = $company;
	}

	public function sc_ajax_handler() {
		$handler      = $_REQUEST['handler'] ?? '';
		$request_data = $_REQUEST['data'] ?? array();
		switch ( $handler ) {
			case 'search_shortcode':
				$output = array();
				switch ( $_REQUEST['request'] ) {
					case 'saleclass':
						unset( $_REQUEST['params']['model'] );
						unset( $_REQUEST['params']['trim'] );
						$_REQUEST['params']['make'] = '';
						break;
					case 'make':
						unset( $_REQUEST['params']['trim'] );
						$_REQUEST['params']['model'] = '';
						break;
					case 'model':
						$_REQUEST['params']['trim'] = '';
				}
				if ( 'Certified' === $_REQUEST['params']['saleclass'] ) {
					$_REQUEST['params']['saleclass'] = 'Used';
					$_REQUEST['params']['certified'] = 'Yes';
				}
				if ( ! empty( $_REQUEST['params'] ) && 'search' !== $_REQUEST['request'] ) {
					$i = 1;
					do {
						$i = 0;
						foreach ( $_REQUEST['params'] as $key => $value ) {
							if ( 'search' !== $key && 'saleclass' !== $key && 'certified' !== $key ) {
								$output['content']['dropdown'][ $key ] = ( new Fn_Inventory() )->cdp_get_db_dropdown_data( $key, $_REQUEST['params'] );

								if ( $key === $_REQUEST['request'] && empty( $output['content']['dropdown'][ $key ]['needle'] ) ) {
									$output['content']['dropdown'][ $key ]['needle'] = ( isset( $_REQUEST['params'][ $key ] ) ? $_REQUEST['params'][ $key ] : '' );
								}

								if ( $output['content']['dropdown'][ $key ]['needle'] && empty( $_REQUEST['params'][ $key ] ) ) {
									$_REQUEST['params'][ $key ] = $output['content']['dropdown'][ $key ]['needle'];
									switch ( $key ) {
										case 'make':
											$_REQUEST['params']['model'] = '';
											break;
										case 'model':
											$_REQUEST['params']['trim'] = '';
											break;
									}
									$i = 1;
								}
							}
						}
					} while ( 0 !== $i );

				}
				$output['url'] = ( new Fn_Inventory() )->cdp_construct_url( $_REQUEST['params'] );
				break;
			case 'get_gform':
				$output['id'] = $_REQUEST['params']['key'];
				( new Saved_Ajax() )->apply_ajax_gravity_hooks( $_REQUEST['params']['hooks'] );
				$form              = gravity_form( $_REQUEST['params']['form'], $display_title = false, $display_description = false, $display_inactive = false, $field_values = false, $ajax = true, $tabindex = 0, $echo = false );
				$output['content'] = '<div class="list-form-wrapper active">' . $form . '</div>';
				break;
			case 'get_vehicle_details':
				$output['id'] = $_REQUEST['params']['key'];
				$params       = array( 'vin' => $_REQUEST['params']['key'] );
				$vehicle_data = $this->get_vehicle_inventory( $params );

				$vin = ( isset( $params['vin'] ) ? array( 'vin' => $params['vin'] ) : '' );
				// db vehicle
				$db_vehicle = ( $vin ? ( new Requests() )->cdp_db_init( 'inventory', $vin )[0] : array() );

				// get photos, standard_equipment, dealer_options from api and merge into db vehicle.
				$response                          = Remote_Requests::http_get_file( '/' . $db_vehicle['dealer_id'] . '/inventory/vehicles.json?vin=' . $db_vehicle['vin'] );
				$response_body                     = json_decode( wp_kses_data( $response['body'] ) );
				$api_vehicle['dealer_options']     = $response_body->dealer_options ?? '';
				$api_vehicle['standard_equipment'] = $response_body->standard_equipment ?? array();
				$api_vehicle['photos_raw']         = $response_body->photos ?? array();
				$vehicle                           = array_merge( $db_vehicle, $api_vehicle );
				switch ( $_REQUEST['params']['tag'] ) {
					case 'info':
						$flags             = array( 'show_eq' => $this->options['theme']['inventory_display']['detail']['general']['show_standard_eq'] );
						$output['content'] = ( new Sc_Views() )->get_sc_detail_vehicle_info_display( $vehicle, $flags );
						break;
					case 'images':
						$output['content'] = ( new Sc_Views() )->get_sc_detail_images_display( $vehicle );
						break;
					default:
						$output['content'] = 'ERROR: Requesting selected inventory. Vehicle may be sold or is no longer in the system.';
						break;
				}

				break;
			case 'get_more_vehicles':
				$atts    = $_REQUEST['params']['atts'];
				$form    = ( isset( $atts['form'] ) ? $atts['form'] : '' );
				$display = ( isset( $atts['display'] ) ? '_' . $atts['display'] : '' );
				unset( $atts['form'] );
				if ( 'certified' === $atts['saleclass'] ) {
					$atts['saleclass'] = 'used';
					$atts['certified'] = 'yes';
				}
				$vehicle_data      = $this->get_vehicle_inventory( $atts );
				$output['content'] = '';
				if ( is_array( $vehicle_data ) && ! empty( $vehicle_data ) ) {
					foreach ( $vehicle_data as $vehicle ) {
						$caller             = 'get_sc_detail' . $display . '_vehicles';
						$output['content'] .= ( new Sc_Views() )->$caller( $vehicle, $this->company, $this->options, $this->options['theme']['overrides']['custom_contact'], $form );
					}
				} elseif ( empty( $vehicle_data ) ) {
					$output['content'] = '<div id="cdp-ajax-end">All Vehicles have been loaded.</div>';
				} else {
					$output['content'] = 'ERROR: Requesting selected inventory. Vehicle may be sold or is no longer in the system.';
				}
				break;
			case 'update_dropdowns':
				$atts = $_REQUEST['params']['atts'];
				if ( 'certified' === $atts['saleclass'] ) {
					$atts['saleclass'] = 'used';
					$atts['certified'] = 'yes';
				}
				switch ( $_REQUEST['params']['filter'] ) {
					case 'saleclass':
						$stack = array( 'make', 'model', 'trim' );
						break;
					case 'make':
						$stack = array( 'model', 'trim' );
						break;
					case 'model':
						$stack = array( 'trim' );
						break;
					default:
						$stack = array(); // Trim Doesn't require a DD Update
						break;
				}
				$output = $this->dropdown_handler( $stack, $atts );
				break;
			default:
				$output['id']    = '';
				$output['error'] = 'That is not a valid FN parameter. Please check your string and try again.';
				break;
		}

		$output = wp_json_encode( $output );
		echo $output;
		wp_die();
	}

	public function get_vehicle_inventory( $params ) {
		$this->vms->tracer = 'Obtaining requested inventory. Ajax from SC Detail.';
		$inventory         = $this->vms->get_inventory( array( 'parameters' => $params ) );

		return $inventory;
	}

	public function dropdown_handler( $requests, $atts ) {
		$alt_id = $atts['dealer_id'] ?? 0;
		if ( ! empty( $requests ) ) {
			foreach ( $requests as $request ) {
				switch ( $request ) {
					case 'make':
						$data = ( new Fn_Inventory() )->cdp_generate_make_options( $this->vms, $atts['saleclass'], $this->options['theme']['overrides']['makes_new']['included'], false, $alt_id );
						if ( isset( $data['val'] ) ) {
							$atts['make']           = $data['val'];
							$results['make']['att'] = $data['val'];
						}
						$results['make']['display'] = $data['display'];
						break;
					case 'model':
						$data = ( new Fn_Inventory() )->cdp_generate_model_options( $this->vms, $atts['saleclass'], $atts['make'], false, $alt_id );
						if ( isset( $data['val'] ) ) {
							$atts['model']           = $data['val'];
							$results['model']['att'] = $data['val'];
						}
						if ( isset( $data['disabled'] ) ) {
							$results['model']['disabled'] = $data['disabled'];
						}
						$results['model']['display'] = $data['display'];
						break;
					case 'trim':
						$data = ( new Fn_Inventory() )->cdp_generate_trim_options( $this->vms, $atts['saleclass'], $atts['make'], $atts['model'], false, $alt_id );
						if ( isset( $data['val'] ) ) {
							$atts['trim']           = $data['val'];
							$results['trim']['att'] = $data['val'];
						}
						if ( isset( $data['disabled'] ) ) {
							$results['trim']['disabled'] = $data['disabled'];
						}
						$results['trim']['display'] = $data['display'];
						break;
				}
			}
		}

		return $results;
	}

}
