<?php

namespace CarDealerPress\Includes\Functions;

use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\includes\helpers\helpers as helpers;
use CarDealerPress\Includes\Helpers\Option;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Show_Theme {
	use Logger;


	public $theme_options;
	public $admin_options;
	public $company;
	private $compare_vehicles;
	private $saved_vehicles;
	private $print_page;
	private $autocheck_flag;
	private $parameters;
	/**
	 * @var int|mixed|string
	 */
	private $autocheck_vin;
	/**
	 * @var array|null|string
	 */
	private $taxonomy;
	/**
	 * @var VehicleManagementSystem
	 */
	private $vms;
	public $db_requests;
	public $db_vehicle;

	public function __construct() {
		$this->set_option_variables();
		$this->vms = new VehicleManagementSystem( $this->admin_options['admin']['help']['host'], helpers::company_id() );
		$this->queue_templates();
	}

	public function set_option_variables() {
		$this->theme_options = Option::get_cdp_theme_options();
		$this->admin_options = Option::get_cdp_admin_options();
		$this->db_requests   = new Requests();
		$db_company          = $this->db_requests->cdp_company_request();
		if ( $db_company ) {
			$this->company = $db_company[0];
		} else {
			$this->company = array();
		}
	}


	public function queue_templates() {
		add_action( 'template_redirect', array( &$this, 'routes' ), 100 );
	}

	public function routes() {
		global $wp_query;
		if ( 'on' === $this->admin_options['admin']['general']['debug_plugin_info'] ) {
			$this->log_error_page();
		}
		$this->taxonomy = ( isset( $wp_query->query_vars['taxonomy'] ) ) ? $wp_query->query_vars['taxonomy'] : null;
		// todo future vms is legacy, should find another way to check for inventory, maybe db check?
		if ( $this->vms ) {
			switch ( $this->taxonomy ) {
				case 'inventory':
					if ( $this->admin_options['admin']['help']['host'] ) {

						$this->parameters = $this->get_parameters();
						$this->fix_bad_wordpress_assumption();

						$current_theme = apply_filters( 'cdp_theme_name', $this->theme_options['theme']['general']['name'] );
						$theme_path    = CDP_DIR_PATH . 'templates/inventory/' . $current_theme;
						$theme_path    = apply_filters( 'cdp_theme_path', $theme_path );

						if ( $this->autocheck_flag ) {
							include_once CDP_DIR_PATH . 'templates/inventory/autocheck.php';
						} elseif ( $this->print_page ) {
							$file = CDP_DIR_PATH . 'templates/inventory/print.php';
							$file = apply_filters( 'cdp_print', $file );
							include_once $file;
						} elseif ( $this->saved_vehicles ) {
							$file = CDP_DIR_PATH . 'templates/inventory/saved.php';
							include_once $file;
						} elseif ( $this->compare_vehicles ) {
							$file = CDP_DIR_PATH . 'templates/inventory/compare.php';
							include_once $file;
						} else {
							if ( isset( $this->parameters['vin'] ) || isset( $this->parameters['id'] ) ) {
								add_action( 'wp_head', array( $this, 'cdp_add_canonical_link' ) );
							}

							if ( $handle = opendir( CDP_DIR_PATH . 'templates/inventory' ) ) {
								while ( false !== ( $file = readdir( $handle ) ) ) {
									include_once CDP_DIR_PATH . 'templates/inventory/invdb-index.php';
								}
								closedir( $handle );
							} else {
								echo __FUNCTION__ . ' Could not open directory at: ' . $theme_path;

								return false;
							}
						}

						$this->stop_wordpress();
					}
					break;
				case 'new-vehicle-sitemap':
				case 'used-vehicle-sitemap':
				case 'video-vehicle-sitemap':
					if ( ! empty( $this->company['seo'] ) ) {
						$sitemap_path = CDP_DIR_PATH . 'templates/sitemap';
						$handle = opendir( $sitemap_path );
						if ( $handle ) {
							while ( false !== ( $file = readdir( $handle ) ) ) {
								if ( 'index.php' === $file ) {
									include_once $sitemap_path . '/index.php';
								}
							}
							closedir( $handle );
						} else {
							echo __FUNCTION__ . ' Could not open directory at: ' . $sitemap_path;

							return false;
						}

						$this->stop_wordpress();
					}
					break;
			}
		}
		// Handle Gravity Form Pop-up.

		if ( isset( $wp_query->query['name'] ) && 'cdp-form-display' === $wp_query->query['name'] && isset( $_GET['form_id'] ) ) {
			include_once CDP_DIR_PATH . 'templates/form-popup.php';
			$this->stop_wordpress();
		}
	}

	public function cdp_add_canonical_link() {
		$rules          = get_option( 'rewrite_rules' );
		$url_rule       = isset( $rules['^(inventory)'] );
		$link_params    = array(
			'year'  => $this->parameters['year'],
			'make'  => $this->parameters['make'],
			'model' => $this->parameters['model'],
			'state' => $this->company['state'],
			'city'  => $this->company['city'],
			'vin'   => $this->parameters['vin'] ?? '',
			'id'    => $this->parameters['id'] ?? '',
		);
		$canonical_link = ( new Fn_Inventory() )->generate_inventory_link( $url_rule, $link_params, '', '', 1 );

		echo '<link href="' . site_url() . $canonical_link . '" rel="canonical" />';
	}

	public function get_parameters() {
		global $wp;
		global $wp_rewrite;

		$permalink_parameters = ! empty( $wp_rewrite->permalink_structure ) ? explode( '/', $wp->request ) : array();
		$server_parameters    = isset( $_GET ) ? array_map( array( &$this, 'sanitize_inputs' ), $_GET ) : null;

		if ( isset( $server_parameters['trim'] ) ) {
			$server_parameters['trim'] = ( $_GET['trim'] ?? '' );
		}
		$parameters             = array();
		$this->compare_vehicles = false;
		$this->saved_vehicles   = false;
		$this->print_page       = false;
		$this->autocheck_flag   = false;

		if ( 'inventory' === $this->taxonomy ) {
			$server_parameters['per_page'] = $this->theme_options['theme']['overrides']['per_page'];

			switch ( $this->theme_options['theme']['overrides']['saleclass'] ) {
				case 'new':
					$server_parameters['saleclass'] = 'New';
					break;
				case 'used':
					$server_parameters['saleclass'] = 'Used';
					break;
				case 'certified':
					$server_parameters['saleclass'] = 'Used';
					$server_parameters['certified'] = 'yes';
					break;
			}
			if ( isset( $permalink_parameters[1] ) && 'autocheck' === $permalink_parameters[1] ) {
				$this->autocheck_flag = true;
				$this->autocheck_vin  = ( isset( $permalink_parameters[2] ) ) ? $permalink_parameters[2] : 0;
			} elseif ( isset( $permalink_parameters[1] ) && 'saved-vehicles' === $permalink_parameters[1] ) {
				$this->saved_vehicles = true;
			} elseif ( isset( $permalink_parameters[1] ) && 'compare-vehicles' === $permalink_parameters[1] ) {
				$this->compare_vehicles = true;
			} else {
				foreach ( $permalink_parameters as $key => $value ) {
					switch ( $key ) {
						case 0:
							$index = 'taxonomy';
							break;
						case 1:
							if ( is_numeric( $value ) ) {
								$index = 'year';
							} else {
								$index = 'saleclass';
							}
							break;
						case 2:
							$index = 'make';
							break;
						case 3:
							$index = 'model';
							break;
						case 4:
							$index = 'state';
							break;
						case 5:
							$index = 'city';
							break;
						case 6:
							if ( 17 === strlen( $value ) ) {
								$index = 'vin';
							} else {
								$index = 'id';
							}
							break;
						default:
							return;
							break;
					}
					$parameters[ $index ] = $value;
				}
				if ( isset( $server_parameters['print_page'] ) ) {
					$this->print_page = true;
				}
			}
		}
		// Need to add both vin and id to parameters.
		if ( isset( $parameters['id'] ) ) {
			$vehicle_id = array( 'id' => $parameters['id'] );
			$db_vehicle = $this->db_requests->cdp_db_init( 'inventory', $vehicle_id );
			if ( ! empty( $db_vehicle ) ) {
				$parameters['vin'] = $db_vehicle[0]['vin'];
			}
		}
		return array_merge( $parameters, $server_parameters );
	}

	public function fix_bad_wordpress_assumption() {
		global $wp_query;
		$wp_query->is_home = false;
	}

	public function stop_wordpress() {
		exit;
	}

	public function log_error_page() {
		if ( function_exists( 'http_response_code' ) ) {
			$page_response = http_response_code();
			if ( 200 !== $page_response ) {
				$page_url      = 'http' . ( isset( $_SERVER['HTTPS'] ) ? 's' : '' ) . '://' . "$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
				$referer       = $_SERVER['HTTP_REFERER'] ?? '';
				$error_message = 'Response Code: ' . $page_response . ' | Page URL: ' . $page_url . ( ! empty( $referer ) ? ' | Referer: ' . $referer : '' );
				$this->append_log( array( 'message' => 'CDP Response Code Error Message (' . CDP_VERSION . ') - ' ) );
				$this->append_log( array( 'message' => $error_message ) );
			}
		}
	}

	public function sanitize_inputs( $input ) {
		if ( is_array( $input ) ) {
			foreach ( $input as $key => $value ) {
				$input[ $key ] = is_scalar( $value )
					? wp_kses_data( $value )
					: array_map(
						array(
							&$this,
							'sanitize_inputs',
						),
						$value
					);
			}
		} else {
			$input = wp_kses_data( $input );
		}

		return $input;
	}


}
