<?php

namespace CarDealerPress\includes\helpers;

use CarDealerPress\Includes\Database\Inventory_Handler;
use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\includes\functions\Logger;
use CarDealerPress\includes\helpers\helpers as helpers;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Multi_Request {
	use Logger;

	private static $wpdb;
	private $curl_url;
	public $company_id;
	private $dealer_id;
	private $current_inventory_version = '5.9.2203.00';
	private $current_company_version   = '6.0.2209.00';
	private $retry_limit               = 5;
	private $retry                     = 0;
	private static $blog               = 1;
	private $admin_options;
	private $theme_options;
	private $inventory_handler;
	/**
	 * @var int
	 */
	private $job_start_time;
	/**
	 * @var string
	 */
	private $api_call;

	public function __construct() {
		global $wpdb;
		self::$wpdb = $wpdb;
		self::$blog = get_current_blog_id();
		$this->set_option_variables();
		$this->inventory_handler = new Inventory_Handler();

		if ( $this->company_id ) {
			$this->disclaimer_check();
		}
	}

	public function set_option_variables() {
		$this->job_start_time = time();
		$this->admin_options  = Option::get_cdp_admin_options();
		$this->theme_options  = Option::get_cdp_theme_options();
		$this->company_id     = helpers::company_id();
		if ( isset( $_POST['cdp_Admin_post_action'] ) && 'dealerChange' === $_POST['cdp_Admin_post_action'] ) {
			if ( isset( $_POST['cdp_Admin_post_new_value'] ) && isset( $_POST['cdp_Admin_post_previous_id'] ) && ! empty( $_POST['cdp_Admin_post_new_value'] ) ) {
				$this->company_id = $_POST['cdp_Admin_post_new_value'];
			}
		}

		$this->api_call = isset( $this->admin_options['admin']['help']['host'] ) ? urldecode( $this->admin_options['admin']['help']['host'] ) : '';
	}



	public function disclaimer_check() {

		$query   = 'Select * from cdp_data where blog_id = "' . self::$blog . '" and data_key = "disclaimer" ';
		$results = self::$wpdb->get_results( $query, ARRAY_A );
		if ( empty( $results ) ) {
			$api_params    = '/' . $this->company_id . '/inventory/vehicles.json?per_page=1';
			$response      = Remote_Requests::http_get_file( $api_params );
			$response_code = wp_remote_retrieve_response_code( $response );
			if ( 200 === $response_code ) {
				$response_data = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
				if ( ! empty( $response_data[0]->disclaimer ) ) {
					$table  = 'cdp_data';
					$data   = array(
						'blog_id'    => self::$blog,
						'data_key'   => 'disclaimer',
						'data_value' => $response_data[0]->disclaimer,
					);
					$format = array( '%d', '%s', '%s' );
					self::$wpdb->insert( $table, $data, $format );
				}
			}
		}
	}

	public function manual_dealer_check( $manual_id ) {

		$results = array();
		if ( $manual_id ) {
			$query       = 'Select * from cdp_data where blog_id = "' . self::$blog . '" and data_key = "dealer" and dealer_id = "' . $manual_id . '"';
			$data_dealer = self::$wpdb->get_results( $query, ARRAY_A );
			if ( empty( $data_dealer ) ) {
				$api_params    = '/api/companies/' . $manual_id;
				$response      = Remote_Requests::http_get_file( $api_params );
				$response_code = wp_remote_retrieve_response_code( $response );
				if ( 200 === $response_code ) {
					$results['found'] = 'Company ID added to company. Will import momentarily.';
				} else {
					$results['error'] = 'Company ID not found. Please enter a different ID.';
				}
			} else {
				$results['error'] = 'Company ID already manually added. Please enter a different ID.';
			}
		} else {
			$results['error'] = 'Company ID not found. Please enter a different ID.';
		}

		return $results;
	}

	public function insert_manual_dealer( $dealer ) {
		if ( ! empty( $dealer ) ) {
			$api_params    = '/api/companies/' . $dealer;
			$response      = Remote_Requests::http_get_file( $api_params );
			$response_code = wp_remote_retrieve_response_code( $response );
			if ( 200 === $response_code ) {
				$this->dealer = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
				if ( $this->dealer ) {
					$this->populate_company_table();
				}
			}
		}
	}

	public function populate_company_table() {

		if ( $this->dealer ) {
			$table_data = array();

			$dealer_id = ( $this->dealer->id ?? 0 );

			$name      = ( $this->dealer->name ?? '' );
			$street    = ( isset( $this->dealer->street ) ? $this->dealer->street : '' );
			$city      = ( isset( $this->dealer->city ) ? $this->dealer->city : '' );
			$state     = ( isset( $this->dealer->state ) ? $this->dealer->state : '' );
			$zip       = ( isset( $this->dealer->zip ) ? $this->dealer->zip : '' );
			$phone     = ( isset( $this->dealer->phone ) ? $this->dealer->phone : '' );
			$country   = ( isset( $this->dealer->country_code ) ? $this->dealer->country_code : '' );
			$automalls = ( isset( $this->dealer->automall_ids ) ? $this->dealer->automall_ids : array() );
			$created   = ( isset( $this->dealer->created ) ? wp_date( 'Y-m-d', strtotime( $this->dealer->created ) ) : '2017-04-01' );
			$updated   = ( isset( $this->dealer->updated ) ? strtotime( $this->dealer->updated ) : time() );
			$runtime   = 0;
			$seo       = ( $this->dealer->seo ?? array() );

			$table_data[] = array(
				'blog_id'                     => self::$blog,
				'company_id'                  => $this->company_id,
				'dealer_id'                   => $dealer_id,
				'name'                        => $name,
				'street'                      => $street,
				'city'                        => $city,
				'state'                       => $state,
				'zip'                         => $zip,
				'phone'                       => $phone,
				'country'                     => $country,
				'automalls'                   => serialize( $automalls ),
				'created'                     => $created,
				'last_updated'                => $updated,
				'inventory_last_updated'      => $runtime,
				'inventory_api_last_modified' => 0,
				'seo'                         => $seo,
			);

			if ( $table_data ) {
				self::$wpdb->insert( 'cdp_companies', $table_data[0] );
			}
		}
	}

	public function init_inventory_auto_update_handler() {
		if ( ! get_transient( 'cdp_auto_dealer_processing_' . self::$blog ) ) {
			set_transient( 'cdp_auto_dealer_processing_' . self::$blog, 'Running', 60 );
			$query_prepare        = 'select * from cdp_companies where blog_id = %d';
			$query_params_prepare = array( self::$blog );
			$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
			$dealers              = self::$wpdb->get_results( $query, ARRAY_A );
			foreach ( $dealers as $key => $dealer ) {
				$automall_check = unserialize( $dealer['automalls'] );
				if ( empty( $automall_check ) ) {
					$api_params = Generic::api_vehicles_params( array( 'dealer_id' => $dealer['dealer_id'] ) );
					$this->append_log( array( 'message' => 'CDP Message: Auto Update Checking Update for Dealer - ' . $dealer['dealer_id'] . ' - params: ' . $api_params ) );
					// Get API Last Modified time and Check Response Code
					$response      = Remote_Requests::http_get_file_headers( $api_params );
					$response_code = wp_remote_retrieve_response_code( $response );
					// First call usually fails
					if ( empty( $response_code ) ) {
						$this->append_log( array( 'message' => 'CDP Message:  Auto Update check, first call failed, trying again in 2 seconds. response_code: ' . $response_code ) );
						sleep( 2 );
						$response      = Remote_Requests::http_get_file_headers( $api_params );
						$response_code = wp_remote_retrieve_response_code( $response );
					}
					switch ( $response_code ) {
						case 200:
							$this->append_log( array( 'message' => 'CDP Message: Auto Update Response 200: Deciding if to update' ) );
							$inventory_api_last_modified = isset( $response['headers']['last-modified'] ) ? strtotime( $response['headers']['last-modified'] ) : 0;
							$this->run_inventory_if_first_time( $dealer );
							$this->run_inventory_if_last_modified_stamps_different( $dealer, $inventory_api_last_modified );
							$this->run_inventory_if_inventory_table_empty( $dealer );
							break;
						case 304:
							$this->append_log( array( 'message' => 'CDP Message: Auto Update Response 304: Not Modified, nothing to do.' ) );
							break;
						case 403:
							$this->remove_suspended_dealer( $dealer );
							break;
						case ' ':
							$this->append_log(
								array(
									'message' => 'CDP Message: CDP Auto Update check failed | ' . $dealer['dealer_id'] . ' | response code: ' . $response_code . '| url: ' . $this->api_call . $api_params,
								)
							);
							break;
						default:
							$this->append_log( array( 'message' => 'CDP Message: Unknown response_code error dealer: ' . $dealer['dealer_id'] . '  response_code:' . $response_code ) );
					}
					$update_values = array(
						'inventory_last_checked'          => time(),
						'inventory_last_checked_response' => $response_code,
					);
					$where_values  = array(
						'blog_id'   => self::$blog,
						'dealer_id' => $dealer['dealer_id'],
					);
					$format        = array( '%d', '%d' );
					self::$wpdb->update( 'cdp_companies', $update_values, $where_values, $format, $format );
				} else {
					$this->append_log( array( 'message' => 'CDP Message: Auto Update has dealer -' . $dealer['dealer_id'] . ' as a main automall account.  Will not update.' ) );
				}
			}
			delete_transient( 'cdp_auto_dealer_processing_' . self::$blog );
		} else {
			// Do Nothing
			$this->append_log( array( 'message' => 'CDP Message: Auto Updater is already running' ) );
		}
	}

	public function remove_suspended_dealer( $dealer ) {
		$query_prepare        = 'select * from cdp_inventory_' . self::$blog . ' where blog_id = %d and company_id = %d and dealer_id = %d';
		$query_params_prepare = array( self::$blog, $this->company_id, $dealer['dealer_id'] );
		$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
		$results              = self::$wpdb->get_results( $query, ARRAY_A );
		if ( empty( $results ) ) {
			$this->append_log( array( 'message' => 'CDP Message: Dealer id: ' . $dealer['dealer_id'] . ' Suspended' ) );
		} else {
			$where        = array(
				'blog_id'    => self::$blog,
				'company_id' => $this->company_id,
				'dealer_id'  => $dealer['dealer_id'],
			);
			$where_format = array( '%d', '%d', '%d' );
			self::$wpdb->delete( 'cdp_inventory_' . self::$blog, $where, $where_format );
			$this->append_log( array( 'message' => 'CDP Message: Dealer id: ' . $dealer['dealer_id'] . ' Suspended, removing inventory' ) );
		}
	}

	public function run_inventory_if_first_time( $dealer ) {
		// Run Inventory if either time stamps are zero
		if ( $dealer['inventory_last_updated'] == 0 || $dealer['inventory_api_last_modified'] == 0 ) {
			$this->append_log( array( 'message' => 'CDP Message: Initial Inventory auto update run | Dealer - ' . $dealer['dealer_id'] ) );
			$this->inventory_handler->init_inventory_handler( $dealer['dealer_id'] );
		}
	}

	public function run_inventory_if_last_modified_stamps_different( $dealer, $inventory_api_last_modified ) {
		// Run Inventory if API Last Modified Stamps don't match
		if ( $dealer['inventory_api_last_modified'] <> $inventory_api_last_modified ) {
			$this->append_log( array( 'message' => 'CDP Message: API Last Modified change | Dealer - ' . $dealer['dealer_id'] . ' | Previous API Last Modified - ' . $dealer['inventory_api_last_modified'] . ' | Current API Last Modified - ' . $inventory_api_last_modified ) );
			$this->inventory_handler->init_inventory_handler( $dealer['dealer_id'] );
		}
	}

	public function run_inventory_if_inventory_table_empty( $dealer ) {
		// Run Inventory if Inventory Table is Empty for Dealer Inventory
		$query_prepare        = 'select count(*) Inv_Total from cdp_inventory_' . self::$blog . ' where blog_id = %d and dealer_id = %d';
		$query_params_prepare = array( self::$blog, $dealer['dealer_id'] );
		$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
		$vehicle_count        = self::$wpdb->get_results( $query, ARRAY_A );
		if ( 0 === $vehicle_count[0]['Inv_Total'] ) {
			$this->append_log( array( 'message' => 'CDP Message: Vehicle Count 0 | Dealer - ' . $dealer['dealer_id'] ) );
			$this->inventory_handler->init_inventory_handler( $dealer['dealer_id'] );
		}
	}

	public function get_url_string() {
		// Initiate curl
		$ch = curl_init();
		// Disable SSL verification
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
		// Will return the response, if false it print the response
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		// Set the url
		curl_setopt( $ch, CURLOPT_URL, $this->curl_url );
		// Execute
		$result = curl_exec( $ch );
		// Closing
		curl_close( $ch );

		return $result;
	}

}
