<?php
/**
 * Constant Class File
 *
 * This file contains Constant class which defines needed.
 */

namespace CarDealerPress\Includes\Init;

use CarDealerPress\includes\helpers\helpers as helpers;
use CarDealerPress\Includes\Helpers\Option;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Constant {

	/**
	 * Define define_constant method in Constant class
	 *
	 * Defines all constants that you need
	 */
	public static function define_constant() {

		/**
		 * CARDEALERPRESS_PATH constant.
		 * Used to specify plugin path
		 */
		if ( ! defined( 'CARDEALERPRESS_PATH' ) ) {
			define( 'CARDEALERPRESS_PATH', trailingslashit( plugin_dir_path( dirname( __FILE__, 2 ) ) ) );
		}

		/**
		 * CARDEALERPRESS_URL constant.
		 * Used to specify plugin urls
		 */
		if ( ! defined( 'CARDEALERPRESS_URL' ) ) {
			define( 'CARDEALERPRESS_URL', trailingslashit( plugin_dir_url( dirname( __FILE__, 2 ) ) ) );
		}

		/**
		 * CDP_IMG constant.
		 * Used to specify image urls inside assets/images directory.
		 */
		if ( ! defined( 'CDP_IMG' ) ) {
			define( 'CDP_IMG', trailingslashit( CARDEALERPRESS_URL ) . 'assets/images/' );
		}

		/**
		 * CDP_CSS constant.
		 * Used to specify css urls inside assets/css directory.
		 */
		if ( ! defined( 'CDP_CSS' ) ) {
			define( 'CDP_CSS', trailingslashit( CARDEALERPRESS_URL ) . 'assets/css/' );
		}

		/**
		 * CDP_JS constant.
		 * Used to specify JS urls inside assets/js directory.
		 */
		if ( ! defined( 'CDP_JS' ) ) {
			define( 'CDP_JS', trailingslashit( CARDEALERPRESS_URL ) . 'assets/js/' );
		}

		/**
		 * CDP_INCLUDES constant.
		 * Used to specify include path inside includes directory.
		 */
		if ( ! defined( 'CDP_INCLUDES' ) ) {
			define( 'CDP_INCLUDES', trailingslashit( CARDEALERPRESS_PATH . 'includes' ) );
		}

		// **
		// * PLUGIN_NAME_LANG constant.
		// * Used to specify language path inside languages' directory.
		// */
		// if ( ! defined( 'PLUGIN_NAME_LANG' ) ) {
		// define( 'PLUGIN_NAME_LANG', trailingslashit( CARDEALERPRESS_PATH . 'languages' ) );
		// }

		/**
		 * PLUGIN_NAME_TPL constant.
		 * Used to specify template urls inside templates' directory.
		 */
		if ( ! defined( 'PLUGIN_NAME_LOGS' ) ) {
			define( 'PLUGIN_NAME_LOGS', trailingslashit( CARDEALERPRESS_PATH . 'logs' ) );
		}

		$blog_id = get_current_blog_id();
		if ( ! defined( 'CDP_LOGS' ) ) {
			define( 'CDP_LOGS', CARDEALERPRESS_PATH . 'logs/cdp-logs-'.$blog_id.'.txt');
		}

		/**
		 * BASE_INVENTORY_API_URL constant
		 */
		if ( ! defined( 'BASE_INVENTORY_API_URL' ) ) {
			// TODO - remove at future point, initially need to help with Transfer of options.
			if ( empty( Option::check_for_option( 'cdp_admin_options', array( 'admin', 'help', 'host' ) ) ) ) {
				define( 'BASE_INVENTORY_API_URL', 'https://api.dealertrend.com' );
			} else {
				define(
					'BASE_INVENTORY_API_URL',
					urldecode(
						Option::check_for_option(
							'cdp_admin_options',
							array(
								'admin',
								'help',
								'host',
							)
						)
					)
				);
			}
		}

		/**
		 * MASTER_FILE constant.
		 */
		if ( ! defined( 'MASTER_FILE' ) ) {
			define( 'MASTER_FILE', CARDEALERPRESS_PATH . '/cardealerpress.php' );
		}

		$plugin_data = array(
			'Name'        => 'Plugin Name',
			'PluginURI'   => 'Plugin URI',
			'Version'     => 'Version',
			'Description' => 'Description',
			'Author'      => 'Author',
			'AuthorURI'   => 'Author URI',
		);

		$plugin_meta = get_file_data( MASTER_FILE, $plugin_data, 'plugin' );

		if ( ! defined( 'CDP_VERSION' ) ) {
			define( 'CDP_VERSION', $plugin_meta['Version'] );
		}

		if ( ! defined( 'PER_PAGE_LIMIT' ) ) {
			define( 'PER_PAGE_LIMIT', 50 );
		}

	}
}
