<?php
/**
 * File for the Help section of the plugin.
 *
 * @package CarDealerPress
 */

namespace CarDealerPress\Templates\Admin\Admin;

use CarDealerPress\Admin\Functions\Main;
use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\includes\helpers\helpers as helpers;
use CarDealerPress\templates\partials\admin\Table_Rows;

/**
 * View for the Help section of the plugin.
 *
 * @package CarDealerPress
 */
function help(): string {
	$home_url           = home_url();
	$inventory_url_new  = "$home_url" . helpers::rewrite_rules()['link_new'];
	$inventory_url_used = "$home_url" . helpers::rewrite_rules()['link_used'];
	$video_check        = ( new Requests() )->cdp_inventory_video_check();
	if ( 0 !== $video_check[0]['Total'] ) {
		$video_site_map_url = '<div><a target="_blank" href="' . $home_url . '/video-vehicle-sitemap.xml">Sitemap Videos</a></div>';
	} else {
		$video_site_map_url = '';
	}
	if ( ! empty( Main::$company['seo'] ) ) {
		$sitemap_information = '<tr><td class="td-full center">
                        <div><a target="_blank" href="' . $home_url . '/new-vehicle-sitemap.xml">Sitemap New</a></div>
                        <div><a target="_blank" href="' . $home_url . '/used-vehicle-sitemap.xml">Sitemap Used</a></div>
                        ' . $video_site_map_url . '
                    </td></tr>';
	} else {
		$sitemap_information = '<tr><td class="td-full center">SEO is not set up.</td></tr>';
	}

	$api_url           = urldecode( BASE_INVENTORY_API_URL );
	$admin_plugins_url = admin_url( 'plugins.php' );
	$cdp_admin_page    = add_query_arg( array( 'page' => 'cardealerpress_admin&tab=general' ), $admin_plugins_url );

	$reset_rewrite            = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'run-flush',
			'class'   => 'do-action-button',
			'value'   => 'Manually Flush Re-Write Rules',
			'ajax_fn' => 'runFlush',
			'message' => 'This will flush the the rewrite rules.',
		)
	);
	$clear_transient          = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'admin-clear-transient',
			'class'   => 'do-action-button',
			'value'   => 'Manually Clear Update Transient',
			'ajax_fn' => 'clearTransient',
			'message' => 'This will clear the transient data.',
		)
	);
	$delete_cron              = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'admin-delete-cron',
			'class'   => 'do-action-button',
			'value'   => 'Manually Delete WP Cron',
			'ajax_fn' => 'deleteCDPCronJob',
			'message' => 'This will delete the cron job that pulls inventory.',
		)
	);
	$truncate_inventory_table = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'admin-truncate-inventory',
			'class'   => 'do-action-button',
			'value'   => 'Reset Inventory Table',
			'ajax_fn' => 'resetInventory',
			'message' => 'This will delete the inventory table.',
		)
	);
	$reset                    = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'cdp-reset-button',
			'class'   => 'do-action-button',
			'value'   => 'Perform Clean Reset',
			'ajax_fn' => 'runCleanReset',
			'message' => 'This will RESET the plugin back to its original default settings, and deactivate the plugin.',
		)
	);
	$alter_tables                    = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'cdp-alter-table-button',
			'class'   => 'do-action-button',
			'value'   => 'Update Company/Inventory Tables',
			'ajax_fn' => 'alterTables',
			'message' => 'This will check and update the database tables for any updates needed.',
		)
	);
	$show_companies_table_columns                    = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'cdp-companies-table-column-button',
			'class'   => 'do-action-button',
			'value'   => 'Show Companies Table Columns',
			'ajax_fn' => 'showCompaniesTableColumns',
			'message' => 'This will output the companies table columns data, to the log file.',
		)
	);
	$show_companies_table                    = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'cdp-companies-table-button',
			'class'   => 'do-action-button',
			'value'   => 'Show Companies Table',
			'ajax_fn' => 'showCompaniesTable',
			'message' => 'This will outpute the companies table data, to the log file.',
		)
	);
	$show_grants                   = table_rows::tr_button_with_confirmation(
		array(
			'id'      => 'cdp-grants-button',
			'class'   => 'do-action-button',
			'value'   => 'Show Grants',
			'ajax_fn' => 'showGrants',
			'message' => 'This will output the grants for the current user, to the log file.',
		)
	);

	return <<<HTML
    <div id="Help" class="tab-content">
        <div id="cdp-content-wrapper" class="settings-wrapper">
            <table class="help-info-box">
                <thead>
                <tr>
                    <th class="th-label"><label><h3>Initial Setup</h3></label></th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td class="td-full center">
                        <p>To get the plugin started you'll need a valid Company ID.</p>
                        <p>A company ID will be provided to you upon purchasing a subscription with 
                          <a href="https://cardealerpress.com" target="_blank">CarDealerPress</a>
                        </p>
                        <p>After you've received a Company ID, you'll need to go to the 
                          <a id="settings-link" href="$cdp_admin_page" title="CarDealerPress API Settings">Admin Settings</a>
                         and fill in the respective company ID field.
                        </p>
                        <p>For a full list of features and tutorials: <a
                                href="https://cardealerpress.com/cardealerpresss-overview/" title="CarDealerPress Help Link"
                                target="_blank">CarDealerPress Features</a>
                        </p>
                        <p>For any further assistance or inventory questions: <a href="https://support.dealertrend.com" 
                        target="_blank" title="CarDealerPress Inventory Help Link">CarDealerPress Help</a>
                        </p>
                    </td>
                </tr>
                </tbody>
            </table>
            <table class="help-info-box">
                <thead>
                <tr>
                    <th class="th-label"><label><h3>Viewing Inventory</h3></label></th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td class="td-full center">
                        <p>If the VMS and Company Feed are both loaded, you may view your inventory here:<br>
                            <div><a href="$inventory_url_new" target="_blank">New Inventory</a></div>
                            <div><a href="$inventory_url_used" target="_blank">Used Inventory</a></div>
                        </p>
                        <p>Please note that any pages or sub-pages that reside at this permalink will no longer be shown.</p>
                    </td>
                </tr>
                </tbody>
            </table>
            <table class="help-info-box">
                <thead>
                <tr>
                    <th class="th-label"><label><h3>Sitemap Links<small>(Requires SEO Helpers)</small></h3></label></th>
                </tr>
                </thead>
                <tbody>
                $sitemap_information
                </tbody>
            </table>
            <table class="help-info-box">
                <thead>
                <tr>
                    <th class="th-label"><label><h3>API URL</h3></label></th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td class="td-full center">
                        <input name="cdp_admin_options[admin][help][host]" type="text" class="cdp-input text-inject"
                               style="width: 200px; margin: 10px 0 0 0;"
                               value="$api_url" disabled/>
                    </td>
                </tr>
                </tbody>
            </table>
            <table class="help-info-box">
                <thead>
                <tr>
                    <th class="th-label"><label><h3>Clean Up and Resets</h3></label></th>
                </tr>
                </thead>
                <tbody>
                    $reset_rewrite
                    $clear_transient
                    $delete_cron
                    $truncate_inventory_table
                    $reset
                </tbody>
            </table>
            <table class="help-info-box">
                <thead>
                <t>
                    <th class="th-label"><label><h3>Debug and Database Information</h3></label>
                    <span>Useful for support issues.</span></th>
                </tr>
                </thead>
                <tbody>
                    $alter_tables
                    $show_companies_table_columns
                    $show_companies_table
                    $show_grants
                </tbody>
            </table>
        </div>
    </div>
HTML;
}
