<?php
namespace CarDealerPress\Templates\Inventory;

use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\includes\helpers\helpers;
use CarDealerPress\Includes\Helpers\Remote_Requests;

if ( $vehicle ) {

	$companies = ( new Requests() )->cdp_db_init( 'company' );
	$company   = array();
	foreach ( $companies as $key => $info ) {
		if ( $info['dealer_id'] === $vehicle['dealer_id'] ) {
			$company = $companies[ $key ];
		}
	}

	$price                    = $fn_inventory->get_price_display(
		array(
			'company' => $this->company,
			'vehicle' => $vehicle,
			'theme'   => 'inventory',
		)
	);
	$vehicle['primary_price'] = $price['primary_price'];

	// TODO future, possible add standard_equip and dealer_options to db.
	$response                      = Remote_Requests::http_get_file( '/' . $vehicle['dealer_id'] . '/inventory/vehicles.json?vin=' . $this->parameters['vin'] );
	$response_body                 = json_decode( wp_kses_data( $response['body'] ) );
	$vehicle['dealer_options']     = $response_body->dealer_options ?? '';
	$vehicle['standard_equipment'] = $response_body->standard_equipment ?? array();
	$vehicle['photos_raw']         = $response_body->photos ?? array();

	$parameters['saleclass'] = $vehicle['saleclass'];
	$fn_inventory->apply_gravity_form_hooks( $vehicle );
	// splide.
	wp_enqueue_script( 'cdp_splide_js', CDP_JS . 'splide/splide.min.js', array( 'jquery' ), '2.4.21', true );
	wp_enqueue_style( 'cdp_splide_style', CDP_CSS . 'splide/splide.min.css', array(), '2.4.21' );
	// Map.
	if ( $this->theme_options['theme']['inventory_display']['detail']['maps']['key'] ) {
		$map_output = $ext_functions->extender_map_generator( $vehicle['dealer_id'] );
	} else {
		$map_output = null;
	}
	$ext_functions->load_parameters( $this->parameters );

	echo $ext_functions->extender_google_tag_script( $vehicle );

	$actual_link = ( isset( $_SERVER['HTTPS'] ) ? 'https' : 'http' ) . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
	?>
	<div id="inventory-wrapper">
		<?php
		if ( $this->theme_options['theme']['inventory_display']['general']['saved_vehicles']['display'] ) {
			echo $ext_functions->extender_saved_vehicle_container();
		}
		?>
		<div id="inventory-detail" itemscope itemtype="https://schema.org/Car" class="inventory-main-container">
			<div id="<?php echo $vehicle['vin'] . '" class="inventory-detail-vehicle-container vehicle-container company-' . helpers::company_id() . ' location-' . $vehicle['dealer_id']; ?>">
				<div id="page-link" class="vehicle-save" style="display: none;" data-name="link" href="<?php echo $actual_link; ?>"></div>
				<div id="inventory-detail-banner">
					<div class="banner-container previous-link" onclick="window.history.go(-1); return false;">
						<span class="previous-icon banner-icon"></span>
						<span class="previous-text banner-text">Previous Search</span>
					</div>
					<div class="banner-container friendly-print"
						 onclick="window.open('?print_page','popup','width=800,height=900,scrollbars=yes,resizable=no,toolbar=no,directories=no,location=no,menubar=yes,status=no,left=0,top=0'); return false">
						<span class="print-icon banner-icon"></span>
						<span class="print-text banner-text">Friendly Print</span>
					</div>
					<div class="banner-container dealer-number">
						<a href="tel: <?php echo $fn_inventory->get_custom_contact_setting( $saleclass, $vehicle['dealer_id'], 'phone' ); ?>">
							<span class="phone-icon banner-icon"></span>
							<span class="phone-text banner-text">Sales: <?php echo $fn_inventory->get_custom_contact_setting( $saleclass, $vehicle['dealer_id'], 'phone' ); ?> </span>
						</a>
					</div>
					<?php
					if ( 'on' === $this->theme_options['theme']['inventory_display']['detail']['general']['social']['facebook']['enable'] ) {
						echo '<div class="banner-container facebook-share">';
						echo '<div class="fb-share-button" data-href="' . $this->page_url . '" data-layout="button" data-size="small" data-mobile-iframe="true">
								<a class="fb-xfbml-parse-ignore" target="_blank" href="https://www.facebook.com/sharer/sharer.php?u=https%3A%2F%2Fdevelopers.facebook.com%2Fdocs%2Fplugins%2F&amp;src=sdkpreparse">Share</a>
								</div>';
						echo '</div>';
					}
					?>
				</div>
				<div id="inventory-detail-top-container">
					<?php
					echo $ext_functions->extender_detail_top_view( $vehicle, $price, $company );
					?>
				</div>
				<div id="inventory-detail-body-container">
					<?php echo $ext_functions->extender_detail_body_view( $vehicle ); ?>
				</div>
				<div id="inventory-detail-bottom-container">
					<?php
					if ( $this->theme_options['theme']['inventory_display']['detail']['general']['display_similar'] ) {
						echo '<div id="similar-container" class="inventory-detail-bottom">' . $ext_functions->extender_similar_vehicle_pull( $vehicle ) . '</div>';
					}
					?>
					<?php echo empty( $map_output ) ? '' : '<div id="map-container" class="inventory-detail-bottom">' . $map_output . '</div>'; ?>
					<?php echo '<div class="inventory-detail-bottom inventory-disclaimer"><p>' . $fn_inventory->get_disclaimer() . '</p></div>'; ?>
				</div>
			</div>
			<?php
			if ( is_active_sidebar( 'vehicle-detail-page' ) ) :
				echo '<div id="inventory-widget-area" >';
				dynamic_sidebar( 'vehicle-detail-page' );
				echo '</div>';
			endif;
			?>
		</div>
		<?php
		if ( $this->theme_options['theme']['inventory_display']['general']['saved_vehicles']['display'] ) {
			echo $ext_functions->extender_compare_vehicle_container();
		}
		?>
	</div>
	<?php
}
?>
