<?php
namespace CarDealerPress\Templates\Inventory;

use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\includes\functions\Fn_Inventory;

$fn_inventory      = new Fn_Inventory();
$this->vms->tracer = 'Obtaining requested inventory print page';

if ( isset( $this->parameters['vin'] ) ) {
	// Currently needed for custom imports on Flamingo.
	$vin                     = ( isset( $this->parameters['vin'] ) ? array( 'vin' => $this->parameters['vin'] ) : array() );
	$vehicle                 = ( $vin ? ( new Requests() )->cdp_db_init( 'inventory', $vin )[0] : array() );
	$contact_info            = array(
		'dealer_id'   => $vehicle['dealer_id'],
		'greeting'    => 'For more information', // TODO need to add an option.
		'dealer_name' => $this->company['name'],
		'phone'       => $this->company['phone'],
	);
	$vehicle['contact_info'] = $contact_info;
}

$type = isset( $vehicle['vin'] ) ? 'detail' : 'list';

if ( 'detail' === $type ) {
	?>
	<style>
		#print-wrapper {
			display: block;
			overflow: hidden;
			clear: both;
			float: none;
			max-width: 500px;
			margin: 0 auto;
		}

		.print-header {
			display: block;
			overflow: hidden;
			clear: both;
			float: none;
			width: 100%;
			text-align: center;
		}

		.print-header-name, .print-header-phone {
			display: block;
			float: left;
			width: 50%;
		}

		.print-vehicle-information {
			width: 60%;
			margin: 5% auto;
		}

		.print-vehicle-information div {
			border-bottom: 2px solid #000;
		}

		.print-vehicle-information span {
			margin: 0 5% 0 0;
		}

		.print-image {
			text-align: center;
			margin: 5% 0;
		}

		.print-price {
			text-align: center;
		}

		.print-description {
			font-size: 12px;
			text-align: center;
			margin: 0 0 5%;
		}

		.print-disclaimer {
			font-size: 10px;
			text-align: center;
			display: block;
			width: 100%;
		}
	</style>

	<div id="print-wrapper">
		<div id="detail-print">
			<div class="print-header">
				<div class="print-headline">
					<h2><?php echo $vehicle['year'] . ' ' . $vehicle['make'] . ' ' . $vehicle['model'] . ' ' . $vehicle['trim']; ?></h2>
				</div>
				<div class="print-header-name"><?php echo $fn_inventory->get_custom_contact_setting( $vehicle['saleclass'], $vehicle['dealer_id'], 'name' ); ?></div>
				<div class="print-header-phone"><?php echo $fn_inventory->get_custom_contact_setting( $vehicle['saleclass'], $vehicle['dealer_id'], 'phone' ); ?></div>
			</div>
			<div class="print-top">
				<div class="print-image">
					<?php
					$list_image = $fn_inventory->no_photo_replace( $vehicle['thumbnail'] );
					echo '<img src="' . $list_image . '" width="300" alt="" title="' . $vehicle['dealer'] . '" />';
					?>
				</div>

				<div class="print-price">
					<?php
					$price = $fn_inventory->get_price_display(
						array(
							'company' => $this->company,
							'vehicle' => $vehicle,
							'theme'   => 'print',
						)
					);
					echo ( ! empty( $price['msrp_text'] ) && 'new' === strtolower( $vehicle['saleclass'] ) ? $price['msrp_text'] : '' ) . '
							' . $price['primary_text'] . $price['compare_text'] . $price['expire_text'];
					?>
				</div>

				<div class="print-vehicle-information">
					<div><span>Stock Number:</span> <?php echo $vehicle['stock']; ?></div>
					<div><span>VIN:</span> <?php echo $vehicle['vin']; ?></div>
					<div><span>Odometer:</span> <?php echo $vehicle['odometer']; ?></div>
					<div><span>Exterior:</span> <?php echo $vehicle['exterior']; ?></div>
					<div><span>Interior:</span> <?php echo $vehicle['interior']; ?></div>
					<div><span>Engine:</span> <?php echo $vehicle['engine']; ?></div>
					<div><span>Transmission:</span> <?php echo $vehicle['transmission']; ?></div>
				</div>
			</div>
			<div class="print-bottom">
				<?php if ( $vehicle['description'] ) { ?>
					<div class="print-description">
						<?php echo $vehicle['description']; ?>
					</div>
				<?php } ?>

				<div class="print-disclaimer">
					<?php echo $fn_inventory->get_disclaimer(); ?>
				</div>
			</div>

		</div>
	</div>

	<?php

} else {

	get_header();
	echo '<h2>Not able to display vehicle print page.</h2>';
	get_footer();

}

?>
