<?php

namespace CarDealerPress\Templates\Sitemap;

$plugin_url = plugins_url( '/cardealerpress' );

use CarDealerPress\Includes\Database\Requests;
use CarDealerPress\includes\functions\Fn_Inventory;
use CarDealerPress\Includes\Helpers\Option;
use DateTime;
use DateTimeZone;

$this->taxonomy = ( isset( $wp_query->query_vars['taxonomy'] ) ) ? $wp_query->query_vars['taxonomy'] : null;
$db_requests    = new Requests();
$fn_inventory   = new Fn_Inventory();
$type           = '';
switch ( $this->taxonomy ) {
	case 'new-vehicle-sitemap':
		$type = 'new';
		break;
	case 'used-vehicle-sitemap':
		$type = 'used';
		break;
	case 'video-vehicle-sitemap':
		$type = 'video';
		break;
}

$site_map_array = array();
$sitemap_output = '';
$date_raw       = wp_date( 'Y-m-d H:i P', strtotime( '-1 day', strtotime( wp_date( 'Y-m-d H:i P', time() ) ) ) );
$utc_timezone   = new DateTimeZone( 'UTC' );
$datetime       = new DateTime( $date_raw, $utc_timezone );
$last_mod       = $datetime->format( 'c' );

if ( 'video' !== $type ) {
	// init.
	array_push( $site_map_array, array( home_url( '/inventory/' . ucfirst( $type ) . '/' ), '0.9' ) );
	// VehicleClass.
	$site_map_db_data = $db_requests->cdp_inventory_special_request( 'saleclass,vehicleclass', ' saleclass="' . $type . '"' );
	if ( $site_map_db_data ) {
		foreach ( $site_map_db_data as $key => $data ) {
			array_push(
				$site_map_array,
				array(
					home_url( '/inventory/' . ucfirst( $type ) . '/?vehicleclass=' . rawurlencode( $data['vehicleclass'] ) ),
					'0.8',
				)
			);
		}
	}
	// Make.
	$site_map_db_data = $db_requests->cdp_inventory_special_request( 'saleclass,make', ' saleclass="' . $type . '"' );
	if ( $site_map_db_data ) {
		foreach ( $site_map_db_data as $key => $data ) {
			$site_map_array[] = array(
				home_url( '/inventory/' . ucfirst( $type ) . '/' . rawurlencode( $data['make'] ) . '/' ),
				'0.8',
			);
		}
	}
	// Certified if Available.
	if ( 'used' === $type ) {
		$site_map_db_data = $db_requests->cdp_inventory_special_request( 'make', ' saleclass="' . $type . '" and certified=1 ' );
		if ( $site_map_db_data ) {
			foreach ( $site_map_db_data as $key => $data ) {
				$site_map_array[] = array(
					home_url( '/inventory/' . ucfirst( $type ) . '/' . rawurlencode( $data['make'] ) . '/?certified=Yes' ),
					'0.8',
				);
			}
		}
	}
	// Model.
	$site_map_db_data = $db_requests->cdp_inventory_special_request( 'saleclass,make,model', ' saleclass="' . $type . '"' );
	if ( $site_map_db_data ) {
		foreach ( $site_map_db_data as $key => $data ) {
			$site_map_array[] = array(
				home_url( '/inventory/' . ucfirst( $type ) . '/' . rawurlencode( $data['make'] ) . '/' . rawurlencode( $data['model'] ) . '/' ),
				'0.7',
			);
		}
	}
	// Trim.
	$site_map_db_data = $db_requests->cdp_inventory_special_request( 'saleclass,make,model,trim', ' saleclass="' . $type . '"' );
	if ( $site_map_db_data ) {
		foreach ( $site_map_db_data as $key => $data ) {
			$site_map_array[] = array(
				home_url( '/inventory/' . ucfirst( $type ) . '/' . rawurlencode( $data['make'] ) . '/' . rawurlencode( $data['model'] ) . '/?trim=' . rawurlencode( $data['trim'] ) ),
				'0.7',
			);
		}
	}
	// Vehicle Detail.
	$site_map_db_data = $db_requests->cdp_inventory_special_request( '', ' saleclass="' . $type . '"' );
	if ( $site_map_db_data ) {
		foreach ( $site_map_db_data as $key => $data ) {
			$site_map_array[] = array(
				home_url( '/inventory/' . $data['year'] . '/' . rawurlencode( $data['make'] ) . '/' . rawurlencode( $data['model'] ) . '/' . rawurlencode( $data['geo_state'] ) . '/' . rawurlencode( $data['geo_city'] ) . '/' . rawurlencode( $data['vin'] ) ),
				'0.7',
				$data['updated'],
			);
		}
	}

	if ( $site_map_array ) {
		foreach ( $site_map_array as $data ) {
			if ( isset( $data[2] ) ) {
				$vehicle_updated = new DateTime( $data[2], $utc_timezone );
				$vehicle_mod     = $vehicle_updated->format( 'c' );
			} else {
				$vehicle_mod = $last_mod;
			}
			$value  = '<url>' . "\n";
			$value .= '<loc>' . $data[0] . '</loc>' . "\n";
			$value .= '<lastmod>' . htmlspecialchars( $vehicle_mod ) . '</lastmod>' . "\n";
			$value .= '<changefreq>daily</changefreq>' . "\n";
			$value .= '<priority>' . $data[1] . '</priority>' . "\n";
			$value .= '</url>' . "\n";

			$sitemap_output .= $value;
		}
		display_output( $sitemap_output, $plugin_url );
	}
} else {
	$site_map_db_data = $db_requests->cdp_inventory_special_request( '', 'video<>""' );
	if ( $site_map_db_data ) {
		foreach ( $site_map_db_data as $key => $data ) {
			$value = '<url>' . "\n";

			$value .= '<loc>' . home_url( '/inventory/' . $data['year'] . '/' . rawurlencode( $data['make'] ) . '/' . rawurlencode( $data['model'] ) . '/' . rawurlencode( $data['geo_state'] ) . '/' . rawurlencode( $data['geo_city'] ) . '/' . rawurlencode( $data['vin'] ) ) . '</loc>' . "\n";

			$value .= '<video:video>' . "\n";
			$title  = ucfirst( $data['saleclass'] ) . ' ' . $data['year'] . ' ' . $data['make'] . ' ' . $data['model'];
			$value .= '<video:thumbnail_loc>' . $data['thumbnail'] . '</video:thumbnail_loc>' . "\n";
			$value .= '<video:title><![CDATA[' . $title . ']]></video:title>' . "\n";
			$value .= '<video:description>' . ( $data['description'] ? '<![CDATA[' . $fn_inventory->cdp_limit_word_count( html_entity_decode( $data['description'] ), 50 ) . ']]>' : $title . ' ' . $data['trim'] ) . '</video:description>' . "\n";
			$value .= '<video:player_loc allow_embed="no"><![CDATA[' . $data['video'] . ']]></video:player_loc>' . "\n";
			$value .= '</video:video>' . "\n";

			$value .= '</url>' . "\n";

			$sitemap_output .= $value;
		}

		display_video( $sitemap_output, $plugin_url );
	}
}


function display_output( $output, $plugin_url ) {
	// header( 'HTTP/1.0 200 OK', true, 200 );
	// Prevent the search engines from indexing the XML Sitemap.
	header( 'X-Robots-Tag: noindex, follow', true );
	header( 'Content-Type: text/xml' );
	header( 'Access-Control-Allow-Origin: *' );
	echo '<?xml version="1.0" encoding="' . get_bloginfo( 'charset' ) . '"?>';
	$stylesheet = '<?xml-stylesheet type="text/xsl" href="' . $plugin_url . '/templates/sitemap/inc/xml-sitemap.xsl' . '"?>';
	echo $stylesheet . "\n";

	echo '<urlset xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ';
	echo 'xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 https://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd" ';
	echo 'xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
	echo $output . "\n";
	echo '</urlset>';

}

function display_video( $output, $plugin_url ) {
	header( 'X-Robots-Tag: noindex, follow', true );
	header( 'Content-Type: text/xml' );
	header( 'Access-Control-Allow-Origin: *' );
	echo '<?xml version="1.0" encoding="' . get_bloginfo( 'charset' ) . '"?>';
	$stylesheet = '<?xml-stylesheet type="text/xsl" href="' . $plugin_url . '/templates/sitemap/inc/xml-sitemap-video-xsl.php"?>';
	echo $stylesheet . "\n";

	echo '<urlset  xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" ';
	echo 'xmlns:video="http://www.google.com/schemas/sitemap-video/1.1">' . "\n";
	echo $output . "\n";
	echo '</urlset>';
}
